# 10183_BOOT OS Kernel

## 概述

10183_BOOT Kernel是由吉林大学三名本科生共同开发的基于[xv6](https://github.com/mit-pdos/xv6-riscv)的RISCV64操作系统内核，提供了部分linux系统调用。

在初赛阶段，我们修改原有的xv6 的文件系统（类似 Unix）并移植了FAT32文件系统，完成了评测所需的全部系统调用。

复赛阶段，我们在第一阶段(7.31日前)完成了

- ext4文件系统
- busybox测试
- 补充部分系统调用

### 系统架构

10183_BOOT包含五个子系统

![arch](docs/img/arch.png "arch")

1. 进程调度（sched）控制着进程对 CPU 的访问。当需要选择下一个进程运行时，由调度程序选择最值得运行的进程。10183boot使用了RR进程调度算法选择新的进程。
2. 内存管理（mm）允许多个进程安全地共享主内存区域。10183boot的内存管理支持虚拟内存，即在计算机中运行的程序，其代码、数据和堆栈的总量可以超过实际内存的大小，操作系统只将当前使用的程序块保留在内存中，其余的程序块则保留在磁盘上。内存管理从逻辑上可以分为硬件无关的部分和硬件相关的部分。硬件无关的部分提供了进程的映射和虚拟内存的对换；硬件相关的部分为内存管理硬件提供了虚拟接口。
3. 虚拟文件系统（Virtul File System，VFS）隐藏了各种不同硬件的具体细节，为所有设备提供了统一的接口，在10183boot上的VFS 支持fat32，ext4文件系统。
4. 网络接口（net）提供了对各种网络标准协议的存取和各种网络硬件的支持。网络接口可分为网络协议和网络驱动程序两部分。网络协议部分负责实现每一种可能的网络传输协议，网络设备驱动程序负责与硬件设备进行通信，每一种可能的硬件设备都有相应的设备驱动程序。
5. 进程间通信（IPC） 支持进程间各种通信机制。

## 环境与配置

- 交叉编译器 riscv64-unknown-elf-gcc
- QEMU模拟器 qemu-riscv64
- RISC-V GCC编译链

### 快速开始

首先克隆 RISC-V GNU Compiler Toolchain的存储库

```bash
git clone --recursive https://github.com/riscv/riscv-gnu-toolchain
```

安装实验编译工具链所需要的包

```bash
sudo apt-get install autoconf automake autotools-dev curl libmpc-dev libmpfr-dev libgmp-dev gawk build-essential bison flex texinfo gperf libtool patchutils bc zlib1g-dev libexpat-dev
```

进入toolchain目录并配置

```bash
cd riscv-gnu-toolchain
./configure --prefix=/usr/local
sudo make
cd ..
```

安装QEMU

```bash
wget https://download.qemu.org/qemu-5.1.0.tar.xz
tar xf qemu-5.1.0.tar.xz
cd qemu-5.1.0
./configure --disable-kvm --disable-werror --prefix=/usr/local --target-list="riscv64-softmmu"
make
sudo make install
cd..
```

下载项目源码

```bash
git clone https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot
```

在QEMU上运行，生成FAT32镜像文件

```bash
make fs
```

运行

```bash
make run
```

## 系统调用和测试

[syscalls说明](https://github.com/oscomp/testsuits-for-oskernel/blob/pre-2024/oscomp_syscalls.md)

系统调用方式遵循RISC-V ABI,即调用号存放在a7寄存器中,6个参数分别储存在a0-a5寄存器 中,返回值保存在a0中。

主要参考[Linux 5.10 syscalls](https://man7.org/linux/man-pages/man2/syscalls.2.html)

### 测试配置、编译、运行

[riscv-syscalls-testing](https://github.com/oscomp/testsuits-for-oskernel/tree/pre-2024/riscv-syscalls-testing)

### 基本功能&文档

- 内存管理
- 进程管理和线程管理
- 异常和系统调用
- [FAT32](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/merge_new_fat_pcj/docs/fat32.md)
- [多核启动流程](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/merge_new_fat_pcj/docs/%E5%A4%9A%E6%A0%B8%E5%90%AF%E5%8A%A8%E6%B5%81%E7%A8%8B.md)
- [如何添加一个系统调用](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/merge_new_fat_pcj/docs/%E5%A6%82%E4%BD%95%E6%B7%BB%E5%8A%A0%E4%B8%80%E4%B8%AA%E7%B3%BB%E7%BB%9F%E8%B0%83%E7%94%A8.md)
- [测试](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/merge_new_fat_pcj/docs/%E6%B5%8B%E8%AF%95syscall.md)

## 初赛文档

见[初赛文档](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/%E5%88%9D%E8%B5%9B%E6%96%87%E6%A1%A3.md)，包括文件结构，在线测试，系统调用

[oscomp/testsuits-for-oskernel (github.com)](https://github.com/oscomp/testsuits-for-oskernel/tree/pre-2024)

## 复赛

### testsuits

[oscomp/testsuits-for-oskernel at final-2024-rv (github.com)](https://github.com/oscomp/testsuits-for-oskernel/tree/final-2024-rv)

### 内核源代码结构

目录

在初赛的基础上进行了调整。

![content](docs/img/content.png "content")

include/目录包含了建立内核代码时所需的大部分包含文件，并按照模块划分。

init/ 子目录包含了内核的初始化代码，这是内核开始工作的起点。

drivers/ 目录包含了内核中所有的设备驱动程序，包括block，net，console。

fs/ 目录包含了所有文件系统的代码，包括ext4，fat32， vfs模块的代码。

net/ 目录包含了内核的网络相关的代码。

mm/ 目录包含了所有的内存管理代码。

ipc/ 目录包含了进程间通信的代码。

kernel/ 目录包含了主内核代码，包括进程管理，原子操作，系统调用。

lib/ 目录包含常用库函数，如string.c
printf.c。

platform/ 目录包含与平台相关的代码，分为qemu部分和硬件部分。

user/ 目录中仅包含init进程构建的依赖。

### 添加功能

#### 线程支持

1. 实现了内核线程kthread（不是内核级线程），每个内核线程有独立的内核栈，共享内核页表，内核线程是内核调度的基本单位。可以为内核添加守护线程，实现动态监控。
2. 基于内核线程，实现了用户线程thread，每个thread由一个kthread支持，每个thread有独立trapframe用来保存用户上下文从而支持用户级线程。
   详细介绍看文档 [线程支持](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/%E7%BA%BF%E7%A8%8B%E6%94%AF%E6%8C%81.md)

#### 信号量

管理线程并发访问共享资源。

见文档 [信号量机制](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/%E4%BF%A1%E5%8F%B7%E9%87%8F%E6%9C%BA%E5%88%B6.md)

#### ext4文件系统

![fs](docs/img/contentFs.png "fs")

见文档[ext4](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/ext4.md)

#### busybox

busybox介绍&实现

[busybox文档](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/busybox.md)

#### lua

[lua文档](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/lua.md)

#### IOzone

[IOzone文档](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/IOzone.md)

### 后续目标

- 补充未完成的功能
- 完善系统

## 项目成员

吉林大学

- 张锶怡 siyizhang.sr@qq.com
- 裴辰举 pcj3195161583@163.com
- 尹健徽 2654901533@qq.com

指导老师

- 胡俊成 jchu@jlu.edu.cn

## 参考

[Thysrael / Exaros · GitLab (eduxiji.net)](https://gitlab.eduxiji.net/Thysrael/exaros)`<br>`
[种田队 / FarmOS · GitLab (eduxiji.net)](https://gitlab.eduxiji.net/202310006101080/zhongtianos)`<br>`
[图漏图森破 / OSKernel2022-X · GitLab (eduxiji.net)](https://gitlab.eduxiji.net/19373469/oskernel2022-x)`<br>`
[HUST-OS/xv6-k210: Port XV6 to K210 board! (github.com)](https://github.com/HUST-OS/xv6-k210)`<br>`
