# busybox

## 功能

BusyBox是一个为Unix类操作系统设计的单一可执行文件，它实现了许多常见的Unix实用程序。这些实用程序包括文件和目录管理、文件传输、文本处理、系统管理、进程管理等。它集成了许多不同的工具，并且这些工具的功能与GNU核心实用程序相似。

主要功能包括：

- 文件管理：`ls`, `cp`, `mv`, `rm`, `touch`等
- 目录管理：`mkdir`, `rmdir`, `pwd`, `cd`等
- 文本处理：`cat`, `echo`, `grep`, `sed`, `awk`等
- 系统管理：`top`, `ps`, `kill`, `init`, `shutdown`等
- 网络工具：`ifconfig`, `ping`, `wget`, `telnet`等

## 实现原理

BusyBox通过一个单一的可执行文件实现多种功能。每个工具在编译时作为函数包含在这个可执行文件中，并通过符号链接或者内建的命令行解析机制来调用相应的功能。这种设计显著减少了系统的整体大小，非常适合嵌入式系统和资源有限的环境。

### 关键实现点

1. **单一可执行文件**：BusyBox的所有工具都被编译成一个单一的可执行文件。这意味着所有的命令共享相同的代码库和全局变量，从而减少了内存和磁盘空间的使用。
2. **符号链接机制**：BusyBox通过创建指向同一个可执行文件的符号链接来提供不同的工具。例如，`/bin/ls`可以是指向BusyBox可执行文件的符号链接，这样当用户运行 `ls`命令时，BusyBox会检测到它是通过 `ls`符号链接调用的，并运行相应的 `ls`函数。
3. **配置和定制**：BusyBox可以通过配置文件进行高度定制，允许用户选择只包含所需的工具和功能。这在嵌入式系统中尤为重要，因为它可以最大限度地减少二进制文件的大小。

## 使用

### 基本用法

安装BusyBox后，可以通过以下方式使用它：

1. **直接调用BusyBox**：可以直接调用 `busybox`命令并指定工具名称，例如：

```sh
  busybox ls
```

2. **通过符号链接**：BusyBox可以通过符号链接提供工具，用户可以像使用普通工具一样使用这些命令。例如，如果 `/bin/ls`是指向BusyBox的符号链接，则可以直接运行 `ls`命令：

```sh
  ls
```

### 常用命令

以下是一些常用的BusyBox命令及其用法：

- **文件管理**：

  ```sh
  busybox ls          # 列出目录内容
  busybox cp file1 file2  # 复制文件
  busybox mv file1 file2  # 移动文件
  busybox rm file     # 删除文件
  ```
- **目录管理**：

  ```sh
  busybox mkdir dir   # 创建目录
  busybox rmdir dir   # 删除目录
  busybox cd dir      # 更改目录
  busybox pwd         # 显示当前目录
  ```
- **文本处理**：

  ```sh
  busybox cat file    # 显示文件内容
  busybox echo "text" # 输出文本
  busybox grep "pattern" file # 在文件中搜索模式
  busybox sed 's/old/new/g' file # 替换文件中的文本
  ```
- **系统管理**：

  ```sh
  busybox top         # 显示系统进程
  busybox ps          # 列出进程
  busybox kill pid    # 杀死进程
  busybox init        # 初始化系统
  busybox shutdown    # 关闭系统
  ```

### 高级用法

BusyBox不仅适用于基本的文件和系统管理任务，还可以用于脚本编写（比如[lua脚本](https://gitlab.eduxiji.net/T202410183992516/oskernel2024-10183boot/-/blob/lwext4/docs/lua.md)）和系统管理。由于它包含了许多常用工具，用户可以在嵌入式系统中使用BusyBox来编写复杂的启动脚本和维护脚本，从而简化系统管理。
