#### ext4文件系统

ext4(Fourth extended filesystem)

ext文件系统一直是许多 Linux 发行版的默认文件系统

#### 特点

1. **更大的文件系统和文件大小支持**：
   EXT4支持更大的文件系统和文件大小。最大单个文件的大小可以达到**16TB**，而整个文件系统的最大大小可以达到**1EB**（1EB = 1024 × 1024TB）。
2. **延迟分配（Delayed Allocation）**：
   延迟分配技术允许文件系统延迟为文件分配磁盘块，直到数据实际被写入（在写入数据时，文件系统不会立即分配磁盘块，而是将数据暂时缓存在内存中，等待更多的数据积累到一起，再一次性分配磁盘块），减少了文件碎片，提高写入性能。
3. **多块分配（Multiblock Allocation）**：
   EXT4引入了多块分配功能，允许同时分配多个块，以减少碎片并提高文件写入的效率。
4. **扩展属性（Extents）**：
   EXT4使用扩展属性来代替**传统的块映射**（传统的块映射表使用一对一的方式记录文件的每一个数据块的位置）。这使得文件可以用一系列**连续的块**来表示（将一系列连续的块记录为一个单元，称为“extent”。每个extent记录一个起始块号和块的数量），减少了碎片并提高了性能。

#### Ext4磁盘布局

![](https://github.com/c-rainstorm/blog/raw/master/res/fs-layout.jpg)

- MBR 为主引导记录用来引导计算机。在计算机启动时，BIOS 读入并执行 MBR，MBR 作的第一件事就是确定活动分区(这对应于双系统的计算机开机时选择启动项，单系统的直接就能确定了所以就不需要选择)，读入活动分区的引导块(Boot block)，引导块再加载该分区中的操作系统。

  分区表(Partition table)用来记录每个分区的起始和结束地址，表中的一个分区为活动分区。
- 每个分区可以装载不同的文件系统，介绍**Ext4**

  - Ex4 文件系统将磁盘分为一系列块组(block groups)。为了减少磁盘碎片带来的性能问题，减少寻道时间，块分配器总是尝试将每个文件的所有块分配到同一个块组中。块组的大小由超级块中的属性来定义，通常是 `8 * block_size_in_bytes` 对于块大小为 4Kb 的磁盘来说，每一个组可以包含 32768 个 block，总共 128Mb。块组的数目为磁盘空间除以块组大小
  - 每个块组的磁盘布局都基本相同，下面就块组 0 做简单介绍
    - 1024 bytes 的 Group 0 Padding（boot block）只有 块组0 有，用于装载该分区的操作系统。
    - 超级块(super block)包含文件系统的所有关键参数。
    - Group Descriptors。用于记录块组内部相关的信息。
    - Reserved GDT Blocks。用于文件系统未来的拓展
    - Data Block Bitmap。用于记录块组内部数据块的使用情况。
    - inode Bitmap。用于记录 inode table 中的 inode 的使用情况。
    - inode table
    - Data Block
  - 在 Ext4 中除日志以外的数据都是以小端法存储的

#### 仓库相关文件/目录

./fs
├── vfs
│   ├── log.c
│   ├── fs.c
│   └── file.c
├── lwext4
│   ├── LICENSE
│   ├── ext4_xattr.c
│   ├── ext4_trans.c
│   ├── ext4_super.c
│   ├── ext4_mkfs.c
│   ├── ext4_mbr.c
│   ├── ext4_journal.c
│   ├── ext4_inode.c
│   ├── ext4_ialloc.c
│   ├── ext4_hash.c
│   ├── ext4_fs.c
│   ├── ext4_extent.c
│   ├── ext4_dir_idx.c
│   ├── ext4_dir.c
│   ├── ext4_debug.c
│   ├── ext4_crc32.c
│   ├── ext4.c
│   ├── ext4_block_group.c
│   ├── ext4_blockdev.c
│   ├── ext4_bitmap.c
│   ├── ext4_bcache.c
│   ├── ext4_balloc.c
│   └── blockdev.c
└── fat32

#### 新加syscalls

```c
extern uint64 sys_lseek(void);
extern uint64 sys_lseek(void);
extern uint64 sys_sendfile(void);
extern uint64 sys_statfs(void);
extern uint64 sys_sysinfo(void);
extern uint64 sys_syslog(void);
extern uint64 sys_utimensat(void);
extern uint64 sys_fstatat(void);
```

参考资料：

[Ext4 Disk Layout - Ext4](https://ext4.wiki.kernel.org/index.php/Ext4_Disk_Layout)

[blog/os/FileSystem-Ext4.md at master · c-rainstorm/blog · GitHub](https://github.com/c-rainstorm/blog/blob/master/os/FileSystem-Ext4.md)

[An introduction to Linux&#039;s EXT4 filesystem | Opensource.com](https://opensource.com/article/17/5/introduction-ext4-filesystem)



#### ext4文件系统适配xv6

扇区组成块，"块"组成块组

块大小：块大小为 4KiB（与 x86 上的页面大小相同，也是块层的默认块大小），但实际大小计算为 2 ^ （10 + `sb.s_log_block_size` ） 字节

![image-20240722225012689](https://cdn.jsdelivr.net/gh/Ywinh/TyporaImages@main/202408161616394.png)

标准块组的布局：

|      | ext4 Super Block | Group Descriptors 组描述符 | Reserved GDT Blocks 保留的 GDT 块 | Data Block Bitmap 数据块位图 | inode Bitmap inode 位图 | inode Table inode 表 | Data Blocks 数据块 |
| ---- | ---------------- | -------------------------- | --------------------------------- | ---------------------------- | ----------------------- | -------------------- | ------------------ |
|      | 1 block          | many blocks                | many blocks                       | 1 block                      | 1 block                 | many blocks          | many more blocks   |



可以使用工具查看官方提供的ext4镜像基本信息：

```
fsstat -f ext4 sdcard-riscv-ext4.img 
```

输出：

```
METADATA INFORMATION
--------------------------------------------
Inode Range: 1 - 131073
Root Directory: 2
Free Inodes: 127759
Inode Size: 256

CONTENT INFORMATION
--------------------------------------------
Block Groups Per Flex Group: 16
Block Range: 0 - 524287
Block Size: 4096
Free Blocks: 52842

BLOCK GROUP INFORMATION
--------------------------------------------
Number of Block Groups: 16
Inodes per group: 8192
Blocks per group: 32768
```



值得注意的：

1. 超级块和块组描述符表，在每个块组中都是重复的（也可以说是backup）
2. 数据块位图跟踪块组中数据块的使用情况。
3. inode 位图记录 inode 表中的哪些条目正在使用中
4. inode 表是 `struct ext4_inode` 线性数组。
5. 表的大小应具有足够的块来存储至少 `sb.s_inode_size * sb.s_inodes_per_group` 字节。
6. 包含 inode 的块组的编号可以计算为 `(inode_number - 1) / sb.s_inodes_per_group` ，该组表中的偏移量为 `index = (inode_number - 1) % sb.s_inodes_per_group` 。*没有 inode 0。*





##### 明确目标

将原本fat32的文件系统修改为ext4。一个文件系统支持的功能无非就是对磁盘中块的读取、写入、查找，在进行这些操作之前我们的os需要对不同的镜像，在内存中初始化一些不同的数据结构来方便查找读写，因此也就有目录项、inode等数据结构，他们在文件系统初始化时同步初始化，作为一些基础的信息。

大致需要修改：

* 原本fat32中，我们是将xv6的inode这一抽象层去除了，使用目录项作为替代，但是这个目录项的结构是和fat32强相关的，比如有他的长目录项、短目录项，他特定的查找方式。为了支持ext4，我们在file结构中，支持ext4特有的数据结构

  ```c
  struct file {
    ...
    int is_dir;
    union {
      struct ext4_file file;
      struct ext4_dir dir;
    };
    ...
  };
  ```

* 修改文件系统初始化函数

* 修改read，write，查找文件的接口函数

* 由于修改了这些接口函数，在syscall中我们不可避免地会使用这些函数，因此需要进一步的适配syscall

反思：这里其实接触到了vfs的概念，也更能在这种繁杂的修改中体会到vfs的好处。假如说我们的接口设计良好，与具体的文件系统格式无关，那么我们至少在syscall中不用在修改函数名和具体行为，这也是比较优雅的设计



##### 具体修改

这里使用大赛推荐的lwext4

1. 首先简单阅读一下lwext里的各个头文件，了解各个模块的接口和作用，重点是要搞清楚如何使用，可以在其提供的test中看下示例
2. 另一个需要解决的问题是如何把lwext4编译进我们的os中，lwext4中存在一些对标准库头文件的使用，比如`#include<stdlib.h>`，我们需要修改为自己实现的头文件
3. 在尽量保持xv6原有抽象层和接口的情况下，将lwext4提供的各种接口包装好，使我们的使用更加方便

