# 引导过程

## OpenSBI

OpenSBI的主要功能就是在M态初始化硬件，然后以S态跳转到内核的位置，完成整个启动过程。此外，OpenSBI作为S态内核的执行环境（EEI），可以以 `ecall` 的方式为S态内核提供一些只有M态才能实现的功能

每个cpu启动时，都在M态。在将控制权交给操作系统时，转换为 S 态



qemu 自带一个 opensbi

> 指定 -bios default 这个选项

由于 opensbi 占据了 0x80000000 到 0x80200000 这段区域，因此我们需要把 xv6 原本这段区域（Kernel）迁移到 0x80200000

加入opensbi后的内存布局

```c
PHYSICAL_MEMORY_TOP-> +----------------------------+----
                      |                            |     
                      |    Free Physical memory    |    
                      |                            |                         
   kernelEnd   -----> +----------------------------+----
                      |                            |                         
                      |          Kernel            |    
                      |                            |                           
 0x8020 0000   -----> +----------------------------+----
                      |                            |                           
                      |         Open SBI           |    
                      |                            |                 
 0x8000 0000  ----->  +----------------------------+----
                      |                            |                           
                      |                            |                           
                      |                            |                           
                      +----------------------------+---
                      |                            |                           
                      |           MMIO             |                
                      |                            |                           
                      +----------------------------+----
                      |                            |                           
                      |                            |                           
                      |                            |                           
                      +----------------------------+                           
```



为了加入 SBI，需要更改的地方

1. `kernel.ld`  里面的内核起始地址
2. `entry.s` 里面，
   * 由于 sbi 把 hart id 放在了 a0 寄存器，xv6 是从 tp 寄存器读出 hart id 的，因此这里需要一个 mv 指令
   * 然后在这里要设置好对应核的 stack，每个核开了 4096 byte，需要设置好 sp 寄存器
3. 然后更改 xv6 的时钟中断实现
   * 因为 xv6 的时钟中断是在 m 态实现的，要实现 sbi 我们一直都会在 S 态，所以利用sbi提供的接口在 s 态实现了时钟中断
   * 更改 `trap.c` 中 `scause` 为时钟中断时的动作



## 多核启动流程

由于 qemu 启动时的启动核不一定是 0，可能是随机的，目前采用的方式是

> 注意这种启动方式可能启动核不是 0，而是其他，后期可能需要更改

* 设置了一个共享变量 `volatile static int boot_hart = -1;`

* 启动核进入 if 判断，初始化自己，然后更改这个共享变量，利用 sbi 来唤醒其他核

  初始化流程：

  * 更改 boot hart

  * 初始化 printf (采用uart)

  * 打印启动信息

  * 初始化物理内存分配

  * 初始化 kernel page table

  * 开启分页

  * 初始化进程

  * 初始化根文件系统 (在proc.c里面)

  * 初始化 trap 并设置好 kernelvec

  * 初始化中断代理 PLIC

  * 初始化 buffer cache

  * 初始化 File table

  * 初始化 virtio 驱动

  * 开启 init 进程

  * 使用 `SBI_HART_START` 启动其他核

    初始化根文件系统

    **开启 `init` 进程**

* 其他核进入else，初始化自己，调用那些 `*inithart`的 函数

  初始化流程

  * 开启分页
  * 设置 kernelvec
  * PLIC 中断代理开启

```c
volatile static int boot_hart = -1;
extern void _entry();
void start_harts();

// start() jumps here in supervisor mode on all CPUs.
void
main()
{
  if(boot_hart == -1){
    boot_hart = cpuid();
    consoleinit();
    printfinit();
    printf("\n");
    printf("xv6 kernel is booting\n");
    printf("\n");
    kinit();         // physical page allocator
    kvminit();       // create kernel page table
    kvminithart();   // turn on paging
    procinit();      // process table
    trapinit();      // trap vectors
    trapinithart();  // install kernel trap vector
    plicinit();      // set up interrupt controller
    plicinithart();  // ask PLIC for device interrupts
    binit();         // buffer cache
    // iinit();         // inode table
    fileinit();      // file table
    virtio_disk_init(); // emulated hard disk
    userinit();      // first user process
    //started = 1;
    __sync_synchronize();
    //启动其他核
    start_harts();
  } else {
    // while(started == 0)
    //   ;
    __sync_synchronize();
    printf("hart %d starting\n", cpuid());
    kvminithart();    // turn on paging
    trapinithart();   // install kernel trap vector
    plicinithart();   // ask PLIC for device interrupts
  }

  set_next_trigger();
  scheduler();        
}

void 
start_harts(){
  for(int i = 0;i < NCPU; i++ ){
    if(sbi_hart_get_status(i) == SBI_HSM_STATE_STOPPED){
      sbi_hart_start(i,(uint64)_entry,0);
    }
  }
}
```



摘自FarmOs:

虽然说一个核就能完成大部分初始化工作，但因为架构要求，多核还是有一些需要各自初始化的代码。多核环境下需要注意这些事项：

- 每个核需要设置独立的页表。允许各个核在运行阶段使用不同的页表
- 每个核有独立的核内时钟，为每个核单独计时
- 每个核有独立的中断处理向量，我们一般将其设为同一个位置
- 每个核有独立的外部中断控制器
- 当一个核与其他核访问同一个变量或同一块内存时，为了缓存能够及时同步，需要加 `fence` 指令，保证之前的所有读写指令在fence之后都已同步到内存。对应gcc的语法为 `__sync_synchronize()`