/*
 * Copyright (c) 2015 Grzegorz Kostka (kostka.grzegorz@gmail.com)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "include/fs/lwext4/ext4_config.h"
#include "include/fs/lwext4/ext4_blockdev.h"
#include "include/fs/lwext4/ext4_errno.h"
#include "include/fs/lwext4/ext4_types.h"
#include "include/block/buf.h"
#include "include/string.h"
#include "include/kernel/proc.h"
#include "include/bio.h"
#include "include/defs.h"

#define BLOCK_SIZE 512

/**********************BLOCKDEV INTERFACE**************************************/
static int blockdev_open(struct ext4_blockdev *bdev);
static int blockdev_bread(struct ext4_blockdev *bdev, void *buf, uint64_t blk_id,
			 uint32_t blk_cnt);
static int blockdev_bwrite(struct ext4_blockdev *bdev, const void *buf,
			  uint64_t blk_id, uint32_t blk_cnt);
static int blockdev_close(struct ext4_blockdev *bdev);
static int blockdev_lock(struct ext4_blockdev *bdev);
static int blockdev_unlock(struct ext4_blockdev *bdev);

/******************************************************************************/
EXT4_BLOCKDEV_STATIC_INSTANCE(blockdev, (uint64)BLOCK_SIZE, (uint64)1 << 22, blockdev_open,
			      blockdev_bread, blockdev_bwrite, blockdev_close,
			      blockdev_lock, blockdev_unlock);
static volatile int isInstantiated = 0;

/******************************************************************************/
static int blockdev_open(struct ext4_blockdev *bdev)
{

	return EOK;
}

/******************************************************************************/

static int blockdev_bread(struct ext4_blockdev *bdev, void *buf, uint64_t blk_id,
			 uint32_t blk_cnt)
{
	// printf("bread!\n");
	uint64 buf_ptr = (uint64)buf;
	// printf("[blockdev_bread] buf:%p\n",buf_ptr);
	for(int i = 0; i < blk_cnt; i++) {
		// printf("blk_id: %d, blk_cnt: %d\n", blk_id + i, blk_cnt);
		struct buf *b = bread(0, blk_id + i);
		memmove((void*)buf_ptr, b->data, BLOCK_SIZE);
		buf_ptr += BLOCK_SIZE;
		brelse(b);
	}
	
	// printf("bread ok!\n");
	return EOK;
}

/******************************************************************************/
static int blockdev_bwrite(struct ext4_blockdev *bdev, const void *buf,
			  uint64_t blk_id, uint32_t blk_cnt)
{
	// printf("bwrite!\n");
	uint64 buf_ptr = (uint64)buf;
	// printf("[blockdev_bwrite] buf:%p\n",buf_ptr);
	for(int i = 0; i < blk_cnt; i++) {
		struct buf *b = bget(0, blk_id + i);
		memmove(b->data, (void*)buf_ptr, BLOCK_SIZE);
		bwrite(b);
		buf_ptr += BLOCK_SIZE;
		brelse(b);
	}
		
	// printf("bwrite ok!\n");
	return EOK;
}
/******************************************************************************/
static int blockdev_close(struct ext4_blockdev *bdev)
{
	/*blockdev_close: skeleton*/
	return EOK;
}

static int blockdev_lock(struct ext4_blockdev *bdev)
{
	/*blockdev_lock: skeleton*/
	return EOK;
}

static int blockdev_unlock(struct ext4_blockdev *bdev)
{
	/*blockdev_unlock: skeleton*/
	return EOK;
}

/******************************************************************************/
struct ext4_blockdev *ext4_blockdev_get(void)
{
	if(isInstantiated == 0){
		isInstantiated = 1;
		EXT4_BLOCKDEV_DYNAMIC_INSTANCE(blockdev, (uint64)BLOCK_SIZE, (uint64)1 << 22, blockdev_open,
					blockdev_bread, blockdev_bwrite, blockdev_close,
					blockdev_lock, blockdev_unlock);
	}
	return &blockdev;
}
/******************************************************************************/

