#ifndef _DEFS_H
#define _DEFS_H

#include "include/riscv.h"
#include "include/fs/vfs/file.h"
struct iovec;
// #include "include/fs/fat32/FileSystem.h"
struct buf;
struct context;
struct file;
// struct inode;
struct pipe;
struct VMA;
struct proc;
struct spinlock;
struct sleeplock;
struct stat;
struct superblock;
struct dirent;
// struct FileSystem;
struct kthread;
struct thread;


// bio.c
void            binit(void);
struct buf*     bread(uint, uint);
void            brelse(struct buf*);
void            bwrite(struct buf*);
void            bpin(struct buf*);
void            bunpin(struct buf*);
// struct buf*     blockRead(FileSystem* fs, u64 blockNum);
// struct buf*     mountBlockRead(FileSystem* fs, u64 blockNum);

//disk.c
void            disk_init(void);
void            disk_read(struct buf *b);
void            disk_write(struct buf *b) ;
void            disk_intr(void);

// console.c
void            consoleinit(void);
void            consoleintr(int);
void            consputc(int);

// exec.c
int             execve(char*, char**,char**);

// file.c
struct file*    filealloc(void);
void            fileclose(struct file*);
struct file*    filedup(struct file*);
void            fileinit(void);
int             fileread(struct file*, uint64, int n);
int             filestat(struct file*, uint64 addr);
int             filewrite(struct file*, uint64, int n,int user);
int             filewritev(struct file *f, struct iovec iov[], int iovcnt);
void            fd_set_exec_flag(struct file *f, int fd, int flag);
int             fd_get_exec_flag(struct file *f, int fd);
// int             faccess(struct dirent *ep, int mode, int flags);
// fs.c
// void            fsinit(int);
// int             dirlink(struct inode*, char*, uint);
// struct inode*   dirlookup(struct inode*, char*, uint*);
// struct inode*   ialloc(uint, short);
// struct inode*   edup(struct inode*);
// void            iinit();
// void            elock(struct inode*);
// void            iput(struct inode*);
// void            eunlock(struct inode*);
// void            eunlockput(struct inode*);
// void            iupdate(struct inode*);
// int             namecmp(const char*, const char*);
// struct inode*   namei(char*);
// struct inode*   nameiparent(char*, char*);
// int             eread(struct inode*, int, uint64, uint, uint);
// void            stati(struct inode*, struct stat*);
// int             writei(struct inode*, int, uint64, uint, uint);
// void            itrunc(struct inode*);


// ramdisk.c
void            ramdiskinit(void);
void            ramdiskintr(void);
void            ramdiskrw(struct buf*);

// kalloc.c
void*           kalloc(void);
void            kfree(void *);
void            kinit(void);

// log.c
// void            initlog(int, struct superblock*);
// void            log_write(struct buf*);
// void            begin_op(void);
// void            end_op(void);

// pipe.c
int             pipealloc(struct file**, struct file**);
void            pipeclose(struct pipe*, int);
int             piperead(struct pipe*, uint64, int);
int             pipewrite(struct pipe*, uint64, int,int);
int             pipewritev(struct pipe *pi, struct iovec iov[], int iovcnt);

// printf.c
void            printf(const char*, ...);
void            panic(char*) __attribute__((noreturn));
void            printfinit(void);

// proc.c
int             cpuid(void);
void            exit(int);
void            thread_exit(int);
int             fork(void);
int             growproc(int);
void            proc_mapstacks(pagetable_t);
pagetable_t     proc_pagetable(struct proc *);
pagetable_t     proc_kpagetable(struct proc* p);
void            proc_freepagetable(pagetable_t, uint64);
void            proc_freekpagetable(pagetable_t kpagetable, uint64 sz);
int             kill(int,int);
int             killed(struct proc*);
int             tkilled(struct thread*t);
void            setkilled(struct proc*);
struct cpu*     mycpu(void);
struct cpu*     getmycpu(void);
struct proc*    myproc();
struct kthread* mykthread();
struct thread*  mythread();
void            procinit(void);
void            scheduler(void) __attribute__((noreturn));
void            sched(void);
void            sleep(void*, struct spinlock*);
void            userinit(void);
int             wait(uint64);
int             wait4(int cpid,uint64 addr,int options);
void            wakeup(void*);
void            yield(void);
int             either_copyout(int user_dst, uint64 dst, void *src, uint64 len);
int             either_copyin(void *dst, int user_src, uint64 src, uint64 len);
void            procdump(void);
uint64          clone(uint64 stack, uint64 flags);
uint64          mmap(uint64 addr, int len, int prot, int flags, int fd, int off);
uint64          munmap(uint64 addr, int len);
uint64          clone_thread(uint64 flags,uint64 stackVa, uint64 ptid, uint64 tls, uint64 ctid);

// swtch.S
void            swtch(struct context*, struct context*);

// spinlock.c
void            acquire(struct spinlock*);
int             holding(struct spinlock*);
void            initlock(struct spinlock*, char*);
void            release(struct spinlock*);
void            push_off(void);
void            pop_off(void);

// sleeplock.c
void            acquiresleep(struct sleeplock*);
void            releasesleep(struct sleeplock*);
int             holdingsleep(struct sleeplock*);
void            initsleeplock(struct sleeplock*, char*);

// string.c
int             memcmp(const void*, const void*, uint);
void*           memmove(void*, const void*, uint);
void*           memset(void*, int, uint);
char*           safestrcpy(char*, const char*, int);
int             strlen(const char*);
int             strncmp(const char*, const char*, uint);
char*           strncpy(char*, const char*, int);
void            wnstr(wchar *dst, char const *src, int len);
void            snstr(char *dst, wchar const *src, int len);
int             wcsncmp(wchar const *s1, wchar const *s2, int len);
char*           strchr(const char *s, char c);

// syscall.c
void            argint(int, int*);
int             argstr(int, char*, int);
void            argaddr(int, uint64 *);
int             fetchstr(uint64, char*, int);
int             fetchaddr(uint64, uint64*);
void            syscall();

// trap.c
extern uint     ticks;
void            trapinit(void);
void            trapinithart(void);
extern struct spinlock tickslock;
void            usertrapret(void);
void            kerneltrapframedump(uint64 sp);

// uart.c
void            uartinit(void);
void            uartintr(void);
void            uartputc(int);
void            uartputc_sync(int);
int             uartgetc(void);

// vm.c
void            kvminit(void);
void            vminit(pagetable_t pagetable);
void            kvminithart(void);
void            vminithart(pagetable_t pagetable);
void            kvmmap(pagetable_t, uint64, uint64, uint64, int);
void            vmmap(pagetable_t pgtbl, uint64 va, uint64 pa, uint64 sz, int perm);
int             mappages(pagetable_t, uint64, uint64, uint64, int);
pagetable_t     kvmcreate();
pagetable_t     uvmcreate(void);
void            uvmfirst(pagetable_t, uchar *, uint);
uint64          uvmalloc(pagetable_t, uint64, uint64, int);
uint64          uvmdealloc(pagetable_t, uint64, uint64);
uint64          vmdealloc(pagetable_t pagetable, uint64 oldsz, uint64 newsz, int do_free);
int             uvmcopy(pagetable_t, pagetable_t, uint64);
int             u2kvmcopy(pagetable_t old, pagetable_t new, uint64 start, uint64 end);
void            uvmfree(pagetable_t, uint64);
void            uvmunmap(pagetable_t, uint64, uint64, int);
void            uvmclear(pagetable_t, uint64);
pte_t *         walk(pagetable_t, uint64, int);
uint64          walkaddr(pagetable_t, uint64);
int             copyout(pagetable_t, uint64, char *, uint64);
int             copyin(pagetable_t, char *, uint64, uint64);
int             copyinstr(pagetable_t, char *, uint64, uint64);
void            _pteprint(pagetable_t pagetable, int level);
void            pteprint(pagetable_t pagetable,uint64 va, int level);
void            vmprint(pagetable_t pagetable);
int             free_vma(struct proc *p);

// plic.c
void            plicinit(void);
void            plicinithart(void);
int             plic_claim(void);
void            plic_complete(int);

// virtio_disk.c
void            virtio_disk_init(void);
void            virtio_disk_rw(struct buf *, int);
void            virtio_disk_intr(void);

// fat.c
// int             fatInit(FileSystem *fs);
// struct dirent*  dirlookup(struct dirent* entry, char* filename, uint* poff);
// char*           formatname(char* name);
// int             getBlockNumber(struct dirent* entry, int dataBlockNum);
// void            emake(struct dirent* dp, struct dirent* ep, uint off);
// struct dirent*  ealloc(struct dirent* dp, char* name, int attr);
// struct dirent*  edup(struct dirent* entry);
// void            eupdate(struct dirent* entry);
// void            etrunc(struct dirent* entry);
// void            eremove(struct dirent* entry);
// void            eput(struct dirent* entry);
// void            estat(struct dirent* ep, struct stat* st);
// void            elock(struct dirent* entry);
// void            eunlock(struct dirent* entry);
// int             enext(struct dirent* dp, struct dirent* ep, uint off, int* count);
// struct dirent*  ename(int fd, char* path);
// struct dirent*  enameparent(int fd, char* path, char* name);
// int             eread(struct dirent* entry, int user_dst, u64 dst, uint off, uint n);
// int             ewrite(struct dirent* entry, int user_src, u64 src, uint off, uint n);
// struct dirent*  create(int fd, char* path, short type, int mode);

// ulib.c
int memcmp(const void *, const void *, uint);
void *memcpy(void *, const void *, uint);


// number of elements in fixed-size array
#define NELEM(x) (sizeof(x)/sizeof((x)[0]))

// #define debug
#define QEMU
//#define visionfive
#define QEMU_DISK
#endif