#ifndef __FILE_H
#define __FILE_H

#include "include/fs/lwext4/ext4.h"
#include "include/kernel/atomic/sleeplock.h"
#include "include/types.h"

// #define O_RDONLY  0x000
// #define O_WRONLY  0x001
// #define O_RDWR    0x002
// #define	O_APPEND	0x0008		/* set append mode */
// //#define O_CREATE  0x200
// #define O_CREATE  0x40  //不符合fcntl规定
// #define O_TRUNC   0x400

#define	O_DIRECTORY	0x20000		/* fail if not a directory */
#define	O_CLOEXEC	0x10000		/* atomically set FD_CLOEXEC */

#define NDEV 4
// #define NFILE 64 //Number of fd that all process can open

struct file {
  enum { FD_NONE, FD_PIPE, FD_INODE, FD_DEVICE,FD_ENTRY,FD_DUMMY } type;
  int ref; // reference count
  char readable;
  char writable;
  struct pipe *pipe; // FD_PIPE
  // struct inode *ip;  // FD_INODE and FD_DEVICE
  int is_dir;
  union {
    struct ext4_file file;
    struct ext4_dir dir;
  };
  uint64 flagslow;   // low 64 bit of flags
  uint64 flagshigh;   // hign 64 bit of flags
  uint off;          // FD_INODE
  short major;       // FD_DEVICE
  uint64 f_flags;
};

#define major(dev)  ((dev) >> 16 & 0xFFFF)
#define minor(dev)  ((dev) & 0xFFFF)
#define	mkdev(m,n)  ((uint)((m)<<16| (n)))

struct iovec {
	void *iov_base; /* Starting address */
	uint64 iov_len; /* Number of bytes to transfer */
};

// map major device number to device functions.
struct devsw {
  int (*read)(int, uint64, int);
  int (*write)(int, uint64, int);
  int (*writev)(int, struct iovec[], int);
  int (*readv)(int, struct iovec[], int);
};

//保存所读取目录的信息
struct linux_dirent64 {
    uint64 d_ino;    // 索引结点号
    i64 d_off;    // 到下一个dirent的偏移
    unsigned short d_reclen;    // 当前dirent的长度
    unsigned char d_type;    // 文件类型
    char d_name[];    //文件名
};

extern struct devsw devsw[];

#define CONSOLE 1

/* these are defined by POSIX and also present in glibc's dirent.h */
#define DT_UNKNOWN	0
#define DT_FIFO		1
#define DT_CHR		2
#define DT_DIR		4
#define DT_BLK		6
#define DT_REG		8
#define DT_LNK		10
#define DT_SOCK		12
#define DT_WHT		14

#define AT_FDCWD -100

/* for faccessat */
#define R_OK	4		/* Test for read permission.  */
#define W_OK	2		/* Test for write permission.  */
#define X_OK	1		/* Test for execute permission.  */
#define F_OK	0		/* Test for existence.  */  

#endif