#ifndef _FS_H
#define _FS_H
// On-disk file system format.
// Both the kernel and user programs use this header file.
#include "include/fs/vfs/stat.h"
#include "include/fs/vfs/file.h"
#include "include/fs/lwext4/ext4.h"

#define ROOTINO  1   // root i-number
#define BSIZE 512  // block size

#define FSMAGIC 0x10203040

#define NDIRECT 12
#define NINDIRECT (BSIZE / sizeof(uint))
#define MAXFILE (NDIRECT + NINDIRECT)

// On-disk inode structure
struct dinode {
  short type;           // File type
  short major;          // Major device number (T_DEVICE only)
  short minor;          // Minor device number (T_DEVICE only)
  short nlink;          // Number of links to inode in file system
  uint size;            // Size of file (bytes)
  uint addrs[NDIRECT+1];   // Data block addresses
};

// Inodes per block.
#define IPB           (BSIZE / sizeof(struct dinode))

// Block containing inode i
#define IBLOCK(i, sb)     ((i) / IPB + sb.inodestart)

// Bitmap bits per block
#define BPB           (BSIZE*8)

// Block of free map containing bit for block b
#define BBLOCK(b, sb) ((b)/BPB + sb.bmapstart)

// Directory is a file containing a sequence of dirent structures.
#define DIRSIZ 14

// struct dirent {
//   ushort inum;
//   char name[DIRSIZ];
// };

int                ext4_init(void);
int                dir_open(struct ext4_dir *dir,const char *path);
int                dir_close(struct ext4_dir *dir);
int                fclose(struct ext4_file *file);
int                fname(struct ext4_file *f, char *path);
int                fopen(struct ext4_file *file, const char *path, uint32_t flags);
int                fread(struct ext4_file *file, uint64 buf, uint off, uint size, int *rcnt, int user);
int                fseek(struct ext4_file *file, uint off, uint origin);
int                fkstatat(char *path, struct stat *kst, int dirfd);
int                funlink(const char *path);
int                fwrite(struct ext4_file *file, uint64 buf, uint off, uint size, int *wcnt,int user);
int                readdir(const char *path);
int                fwritev(struct ext4_file *f, struct iovec* iov, int iovcnt, uint off);
int                freadv(struct ext4_file *f, struct iovec* iov, int iovcnt, uint off);
uint32             get_inode_type(struct ext4_file *file);
uint32             fflag_to_oflag(uint fflag);
uint32             oflag_to_fflag(uint oflag);
int                mkdir(const char *path);
int                is_relative(const char *path);
char*              to_abspath(char *path);


#endif