#ifndef _KTHREAD_H
#define _KTHREAD_H
#include "include/ipc/signal.h"
#include "include/kernel/context.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/riscv.h"

#define KTHREAD_NUM 1000
enum kthreadstate { kt_UNUSED, kt_USED, kt_SLEEPING, kt_RUNNABLE, kt_RUNNING, kt_ZOMBIE };


typedef struct kthread {
  struct spinlock lock;

  // kt->lock must be held when using these:
  enum kthreadstate state;        // kthread state
  void *chan;                  // If non-zero, sleeping on chan
  int xstate;                  // Exit status to be returned to parent's wait
  int ktid;                     // ktread ID
  struct proc* p;               // 这个内核线程属于哪一个进程
  struct thread *t;             //这个内核线程属于哪一个用户线程
  // wait_lock must be held when using this:

  // these are private to the kthread, so kt->lock need not be held.
  uint64 kstack;               // Virtual address of kernel stack
  uint64 kstack_pa;            // 内核栈的物理地址，方便释放
  pagetable_t kpagetable;      // Kernel page table
  struct context context;      // swtch() here to run kthread
  char name[16];               // kthread name (debugging)
  struct kthread *next;
  struct kthread *prev;

}kthread;

void kthreadinit();
kthread *allockthread();
void createkthread(void(*func)(),char*name);
void freekthread(kthread*kt);

#endif