#ifndef _PROC_H
#define _PROC_H
#include "include/ipc/signal.h"
#include "include/kernel/context.h"
#include "include/kernel/thread.h"
#include "include/fs/lwext4/ext4.h"

// Per-CPU state.
struct cpu {
  kthread* kthread;
//  struct proc *proc;          // The process running on this cpu, or null.
  struct context context;     // swtch() here to enter scheduler().
  int noff;                   // Depth of push_off() nesting.
  int intena;                 // Were interrupts enabled before push_off()?
};

extern struct cpu cpus[NCPU];


enum procstate { UNUSED, USED, SLEEPING, RUNNABLE, RUNNING, ZOMBIE };

typedef struct rlimit {
  uint64 rlim_cur;
  uint64 rlim_max;
}rlimit;

//mmap需要都数据结构
#define VMA_MAX 16
struct VMA{
   int valid;        //有效位，当值为 0 时表示无效，即为 empty element
   uint64 addr;      //记录起始地址
   int len;          //长度
   int prot;         //权限（read/write）
   int flags;        //区域类型（shared/private）
   int off;          //偏移量
   struct file* f;   //映射的文件
   uint64 mapcnt;    //（延迟申请）已经映射的页数量
 };

// Per-process state
struct proc {
  struct spinlock lock;

  // p->lock must be held when using these:
  enum procstate state;        // Process state
  int killed;                  // If non-zero, have been killed
  int xstate;                  // Exit status to be returned to parent's wait
  int pid;                     // Process ID

  // wait_lock must be held when using this:
  struct proc *parent;         // Parent process

  // these are private to the process, so p->lock need not be held.
  thread*main_thread;
  thread *thread_queue;
  int thread_num;
  uint64 sz;                   // Size of process memory (bytes)
  pagetable_t pagetable;       // User page table
  struct file *ofile[NOFILE];  // Open files
  struct ext4_dir cwd;           // Current directory
  char name[16];               // Process name (debugging)
  struct VMA vma[VMA_MAX];     // virtual memory address field arr
  uint64 maxaddr;              // virtual memory to be asigned to the vma
  uint64 utime;
  uint64 ktime;
  int filelimit;

  //signal
  sigaction sigaction[SIGRTMAX+1];  //signal action 实时信号

  
};

//clone flags
#define CLONE_VM	0x00000100
//#define CLONE_FS	0x00000200
//#define CLONE_FILES	0x00000400
#define CLONE_SIGHAND	0x00000800
//#define CLONE_PIDFD	0x00001000
//#define CLONE_PTRACE	0x00002000
#define CLONE_VFORK	0x00004000
#define CLONE_PARENT	0x00008000
//#define CLONE_THREAD	0x00010000
//#define CLONE_NEWNS	0x00020000
#define CLONE_SYSVSEM	0x00040000
#define CLONE_SETTLS	0x00080000
#define CLONE_PARENT_SETTID	0x00100000
#define CLONE_CHILD_CLEARTID	0x00200000
#define CLONE_DETACHED	0x00400000
#define CLONE_UNTRACED	0x00800000
#define CLONE_CHILD_SETTID	0x01000000
/*#define CLONE_NEWCGROUP	0x02000000
#define CLONE_NEWUTS	0x04000000
#define CLONE_NEWIPC	0x08000000
#define CLONE_NEWUSER	0x10000000
#define CLONE_NEWPID	0x20000000
#define CLONE_NEWNET	0x40000000
#define CLONE_IO	0x80000000
*/

#endif