#ifndef _Thread_H_
#define _Thread_H_

#include "include/ipc/signal.h"
#include "include/kernel/context.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/riscv.h"
#include "include/kernel/kthread.h"

#define THREAD_NUM 1000

typedef struct thread
{
    struct spinlock lock;

    // kt->lock must be held when using these:
    kthread *kt; // kthread
    int killed;   
    struct proc *p; // 这个线程属于哪一个进程

    // wait_lock must be held when using this:

    // these are private to the kthread, so kt->lock need not be held.
    uint64 vtf;               //trapframe在用户空间的地址
    struct trapframe *trapframe;

    uint64 set_child_tid;
    uint64 clear_child_tid;
    __sigset_t sig_set;            // signal mask
    int sig_pending[SIGRTMAX + 1]; // pending signal
    struct trapframe *sig_tf;      // trapframe for signal
    struct thread *next;
    struct thread *prev;
} thread;

typedef struct thread_stack_param {
  uint64 func_point;
  uint64 arg_point;
}thread_stack_param;

void threadinit();
thread *allocthread(struct proc *p);
void freethread(thread*t,struct proc*p);
void free_all_thread(struct proc *p);
void free_all_thread_except(struct proc *p,thread*t);
void transferspace(pagetable_t old,pagetable_t new,thread*t);

#endif