#ifndef KMM_H_
#define KMM_H_

#include "include/types.h"
#include "include/riscv.h"
#include "include/kernel/atomic/spinlock.h"

#define KMEM_OBJ_MIN_SIZE ((uint64)32)
#define KMEM_OBJ_MAX_SIZE ((uint64)4048)
#define KMEM_OBJ_MAX_COUNT (PGSIZE / KMEM_OBJ_MIN_SIZE)

#define KMEM_TABLE_SIZE 17
#define TABLE_END 255

#define _malloc_allocator() \
    ((struct kmem_allocator *)kmalloc(sizeof(struct kmem_allocator)))

// calculate the capacity
#define _calc_capa(roundup_size) \
    ((PGSIZE - ROUNDUP16(KMEM_NODE_FIX)) / ((roundup_size) + 1))

// the size of fixed part in kmem_node
#define KMEM_NODE_FIX \
    (sizeof(struct kmem_node *) + 2 * sizeof(uint64) + 2 * sizeof(uint8))

// hash map
#define _hash(n) \
    ((n) % KMEM_TABLE_SIZE)

#define ROUNDUP16(n) \
    (((n) + 15) & ~0x0f)

struct kmem_node
{
    struct {
        uint64 obj_size;                 // object size
        uint64 obj_addr;                 // start addr of the first avail object
    } config;
    uint8 avail_cnt;                     // current available count of object
    uint8 alloc_cnt;                     // current allocated count of object 
    uint8 obj_table[KMEM_OBJ_MAX_COUNT]; // linked object table

    struct kmem_node *next;
};

struct kmem_allocator
{
    struct spinlock lock;        // the lock to protect this allocator

    uint obj_size;               // the obj size that allocator allocates
    uint16 npages;
    uint16 nobjs;

    struct kmem_node *list;      // point to first kmem_node
    struct kmem_allocator *next; // point to next kmem_allocator
};

void *kmalloc(uint size);
void kmfree(void* addr);
void *kcalloc(int n, uint size);
void kmallocinit(void);

#endif