#include "include/types.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/init/sbi.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"

volatile static int boot_hart = -1;
extern void _entry();
void start_harts();
int started=0;
extern struct proc proc[NPROC];
extern pagetable_t kernel_pagetable;

extern char __bss_start, __bss_end; // 引用链接器脚本中定义的符号

void clear_bss_section(void) {
    char* bss = &__bss_start;
    char* bss_end = &__bss_end;

    while (bss < bss_end) {
        *bss++ = 0;
    }
}

// start() jumps here in supervisor mode on all CPUs.
void
main()
{
  if(boot_hart == -1){
    boot_hart = cpuid();
    clear_bss_section();
    consoleinit();
    printf("uart success\n");
    printf("boot_hart %d\n",boot_hart);
    printfinit();
    printf("\n");
    printf("xv6 kernel is booting\n");
    printf("\n");
    kinit();         // physical page allocator
    kvminit();       // create kernel page table
    kvminithart();   // turn on paging
    kthreadinit();
    // kmallocinit();
    threadinit();
    procinit();      // process table
    trapinit();      // trap vectors
    trapinithart();  // install kernel trap vector
    plicinit();      // set up interrupt controller
    plicinithart();  // ask PLIC for device interrupts
    binit();         // buffer cache
    // iinit();         // inode table
    fileinit();      // file table
    disk_init(); // emulated hard disk
    userinit();      // first user process
    started = 1;
    __sync_synchronize();
    //启动其他核
  // start_harts();

  } else {
    while(started == 0)
      ;
    __sync_synchronize();
    printf("hart %d starting\n", cpuid());
    kvminithart();    // turn on paging
    trapinithart();   // install kernel trap vector
    plicinithart();   // ask PLIC for device interrupts
  }

  set_next_trigger();

  scheduler();        
}

void 
start_harts(){
  for(int i = 0;i < NCPU; i++ ){
    if(sbi_hart_get_status(i) == SBI_HSM_STATE_STOPPED){
      sbi_hart_start(i,(uint64)_entry,0);
    }
  }
}