#include "include/types.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/fs/vfs/fcntl.h"
#include "include/ipc/signal.h"
#include "include/string.h"



int set_sigaction(int signum, sigaction const *act, sigaction *oldact) {

  struct proc *p = myproc();
  if (p->sigaction[signum].__sigaction_handler.sa_handler != NULL &&
      oldact != NULL) {
    *oldact = p->sigaction[signum];
  }
  if (act != NULL) {
    p->sigaction[signum] = *act;
  }
  return 0;
}

int sigprocmask(int how, __sigset_t *set, __sigset_t *oldset) {
//  struct proc *p = myproc();
  thread*t=mythread();

  for (int i = 0; i < SIGSET_LEN; i++) {
    if (oldset != NULL) {
      oldset->__val[i] = t->sig_set.__val[i];
    }
    if (set == NULL)
      continue;
    switch (how) {
    case SIG_BLOCK:
      t->sig_set.__val[i] |= set->__val[i];
      break;
    case SIG_UNBLOCK:
      t->sig_set.__val[i] &= ~(set->__val[i]);
      break;
    case SIG_SETMASK:
      t->sig_set.__val[i] = set->__val[i];
      break;
    default:
      break;
      // panic("invalid how\n");
    }
  }
  // SIGTERM SIGKILL SIGSTOP cannot be masked
  t->sig_set.__val[0] &= 1ul << SIGTERM | 1ul << SIGKILL | 1ul << SIGSTOP;
  return 0;
}

uint64 rt_sigreturn(void) {
//  struct proc *p = myproc();
  thread*t=mythread();
  memmove(t->trapframe, t->sig_tf, sizeof(struct trapframe));
  kfree(t->sig_tf);
  t->sig_tf = 0;
  return t->trapframe->a0;
  return 0;
}

void sighandle(void) {
  struct proc *p = myproc();
  thread*t=mythread();
  // debug_print("sighandle %p a0:%d a7:%d\n", p->sig_pending.__val[0],
  // p->trapframe->a0, p->trapframe->a7);
  for(int i=0;i<SIGRTMAX+1;i++){
    if(t->sig_pending[i]&&!(t->sig_set.__val[0]&(1ul<<i))){//信号是否被block
    __sighandler_t hander=p->sigaction[i].__sigaction_handler.sa_handler;
    if(hander==SIG_DFL){
      acquire(&t->lock);
        t->killed=1;
      release(&t->lock);
    }else if(hander==SIG_IGN){
      continue;
    }else if(hander!=NULL){
      t->sig_tf = kalloc();
    memmove(t->sig_tf, t->trapframe, sizeof(struct trapframe));
    t->trapframe->epc =
        (uint64)p->sigaction[i].__sigaction_handler.sa_handler;
    t->trapframe->ra = (uint64)SIGTRAMPOLINE;
    t->trapframe->sp = t->trapframe->sp - PGSIZE;
    t->sig_pending[i] =0;
    break;
    }
    }
  }
}