#include "include/types.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/riscv.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/defs.h"
#include "include/kernel/elf.h"
#include "include/fs/lwext4/ext4.h"
#include "include/fs/vfs/fs.h"
// #include "include/fs/fat32/fat32.h"
// #define DEBUG

//动态链接器所需要的一些辅助信息数组（Auxiliary Vector)
// type: 类型 val:类型对应的值
#define ADD_AUXV(type, val) \
  aux[index++] = type; \
  aux[index++] = val;


//动态链接器所需要的一些辅助信息数组（Auxiliary Vector)
// type: 类型 val:类型对应的值
#define ADD_AUXV(type, val) \
  aux[index++] = type; \
  aux[index++] = val;

static int loadseg(pde_t *, uint64, struct ext4_file *, uint, uint);

int flags2perm(int flags)
{
    int perm = PTE_A|PTE_D;
    if(flags & 0x1)
      perm |= PTE_X;
    if(flags & 0x2)
      perm |= PTE_W;
    return perm;
}
/*进程在执行exec系统调用时，所有除了调用该系统调用的线程外的其余线程全部消失*/
int
execve(char *path, char **argv,char **envp)
{
  #ifdef DEBUG
    printf("entering exec\n");
    printf("[exec] path is %s\n",path);
  #endif
  char *s, *last;
  int i, off, r = 0, rcnt = 0, index=0;
  uint64 argc, envc,sz = 0, sp, ustack[MAXARG], estack[MAX_ENV+1], stackbase;
  struct elfhdr elf;
  // struct inode *ip;
  struct ext4_file f;
  struct proghdr ph;
  pagetable_t pagetable = 0, oldpagetable;
  struct proc *p = myproc();
  thread* t=mythread();

  // begin_op();
  if(is_relative(path))
      path = to_abspath(path);
  #ifdef debug
    printf("[execve] path: %s\n", path);
  #endif

  if(( r = fname(&f,path)) != EOK){
    // end_op();
    printf("path is %s not find de\n",path);
    return -1;
  }
  #ifdef DEBUG
    printf("[exec] find de and name is %s\n",de->filename);
  #endif

  // Check ELF header
  r = fread(&f, (uint64)&elf, 0, sizeof(elf), &rcnt, 0);
  printf("elf addr %p\n",&elf);
  if(r != EOK || rcnt != sizeof(elf))
    goto bad;

  if(elf.magic != ELF_MAGIC)
    goto bad;

  if((pagetable = proc_pagetable(p)) == 0)
    goto bad;
  
  // Load program into memory.
  for(i=0, off=elf.phoff; i<elf.phnum; i++, off+=sizeof(ph)){
    r = fread(&f,(uint64)&ph,off,sizeof(ph),&rcnt,0);
    if(r != EOK || rcnt != sizeof(ph))
      goto bad;
    if(ph.type != ELF_PROG_LOAD)
      continue;
    if(ph.memsz < ph.filesz)
      goto bad;
    if(ph.vaddr + ph.memsz < ph.vaddr)
      goto bad;
    // if(ph.vaddr % PGSIZE != 0)
    //   goto bad;
    uint64 sz1;
    // 为虚拟地址分配物理内存
    if((sz1 = uvmalloc(pagetable, sz, ph.vaddr + ph.memsz, flags2perm(ph.flags))) == 0)
      goto bad;
    sz = sz1;
    // 把segment加载到刚分配的物理内存
    if(loadseg(pagetable, PGROUNDDOWN(ph.vaddr), &f, PGROUNDDOWN(ph.off), ph.filesz+(ph.vaddr-PGROUNDDOWN(ph.vaddr))) < 0)
      goto bad;
  }
  // eunlock(de);
  // eput(de);
  fclose(&f);
  // end_op();
  // ip = 0;
  p = myproc();
  uint64 oldsz = p->sz;
  sz=PGROUNDUP(sz);
  uint64 sz1;
  // Allocate two pages at the next page boundary.
  // Make the first inaccessible as a stack guard.
  // Use the second as the user stack.
  if((sz1=uvmalloc(pagetable,sz, sz+4*PGSIZE, PTE_W)) == 0) //分配的栈是否需要更大
    goto bad;
  // 清除保护页的PTE权限
  sz=sz1;
  uvmclear(pagetable, sz - 4 * PGSIZE);
  sp = sz;
  stackbase = sp - 3 * PGSIZE;
  stackbase = sp - 3 * PGSIZE;

  printf("stack range:%p----%p\n",stackbase,sp);

  //清除进程中除当前线程的所有其余线程，并将当前线程转换地址空间
  free_all_thread_except(p,t);
  transferspace(p->pagetable,pagetable,t);

  oldpagetable = p->pagetable;
  p->pagetable = pagetable;
  p->main_thread=t;


  // sp 目前是空栈

  // [ end marker ] 8 (= NULL)
  sp -= 8;

  //将环境变量字符串压入栈, estack 用于存储 pointer 即 envp 向量
  for(envc = 0; envp[envc]; envc++){
    if(envc >= MAX_ENV)
      goto bad;
    sp -= strlen(envp[envc]) + 1;
    sp -= sp % 16; // riscv sp must be 16-byte aligned
    if (sp < stackbase)
      goto bad;
    if (copyout(pagetable, sp, envp[envc], strlen(envp[envc]) + 1) < 0)
      goto bad;
    estack[envc] = sp;
  }
  estack[envc] = 0;
  
  //将argv字符串压入栈, argc是通过ret作为a0参数压入栈的？
  // Push argument strings, prepare rest of stack in ustack.
  for(argc = 0; argv[argc]; argc++){
    if(argc >= MAXARG)
      goto bad;
    sp -= strlen(argv[argc]) + 1;
    sp -= sp % 16; // riscv sp must be 16-byte aligned
    if(sp < stackbase)
      goto bad;
    if(copyout(pagetable, sp, argv[argc], strlen(argv[argc]) + 1) < 0)
      goto bad;
    ustack[argc] = sp;
  }
  ustack[argc] = 0;


  // Load AUX vectors
  sp -= 16;
  uint64_t random[2] = { 0x7be6f23c6eb43a7e, 0xb78b3ea1f7c8db96 };
  if(sp < stackbase || copyout(pagetable, sp, (char*)random, 16) < 0)
    goto bad;
  
  uint64 aux[MAX_AT * 2];

  ADD_AUXV(AT_HWCAP, 0);
  ADD_AUXV(AT_PAGESZ, PGSIZE);
  ADD_AUXV(AT_PHDR, elf.phoff);
  ADD_AUXV(AT_PHENT, elf.phentsize);
  ADD_AUXV(AT_PHNUM, elf.phnum);
  ADD_AUXV(AT_BASE, 0);
  ADD_AUXV(AT_ENTRY, elf.entry);
  ADD_AUXV(AT_UID, 0);
  ADD_AUXV(AT_EUID, 0);
  ADD_AUXV(AT_GID, 0);
  ADD_AUXV(AT_EGID, 0);
  ADD_AUXV(AT_SECURE, 0);
  ADD_AUXV(AT_RANDOM, sp);
  ADD_AUXV(AT_NULL, 0);

  //三个向量的压栈顺序：AUX -> envp -> argv,argc
  // 1. AUX vector, 已经16字节对齐
  sp -= sizeof(aux);
  if(copyout(pagetable, sp, (char*)aux, sizeof(aux)) < 0)
    goto bad;

  // 2.envp vector
  //将环境变量表压入栈
  //  if(envp[0]){
  sp-= (envc+1) * sizeof(uint64);
  // sp-=sp % 16;
  if(sp < stackbase)
    goto bad;
  if(copyout(pagetable,sp,(char*)estack,(envc+1)*sizeof(uint64))<0)
    goto bad;
  //  }
  p->main_thread->trapframe->a2 = sp;
  
  // 3.argv vector
  // push the array of argv[] pointers.
  sp -= (argc+1) * sizeof(uint64);
  // sp -= sp % 16;
  if(sp < stackbase)
    goto bad;
  if(copyout(pagetable, sp, (char *)ustack, (argc+1)*sizeof(uint64)) < 0)
    goto bad;

  // arguments to user main(argc, argv)
  // argc is returned via the system call return
  // value, which goes in a0.
  p->main_thread->trapframe->a1 = sp;
  
  // push the argc.
  sp -= sizeof(uint64);
  if(sp < stackbase)
    goto bad;
  if(copyout(pagetable, sp, (char*)&argc, sizeof(uint64)) < 0)
    goto bad;  



  // print_stack(pagetable,sp);
  // Save program name for debugging.
  for(last=s=path; *s; s++)
    if(*s == '/')
      last = s+1;
  safestrcpy(p->name, last, sizeof(p->name));
  
  p->main_thread->trapframe->epc = elf.entry;  // initial program counter = main
  p->main_thread->trapframe->sp = sp; // initial stack pointer
  p->sz=sz;
  
  proc_freepagetable(oldpagetable, oldsz);
  return argc; // this ends up in a0, the first argument to main(argc, argv)

 bad:
  if(pagetable)
    proc_freepagetable(pagetable, sz);
  if(&f){
    fclose(&f);
  }
  printf("exec bad return\n");
  return -1;
}

// void print_stack(pagetable_t pagetable, uint64 sp){
//   uint64 argc, envc,sz = 0, ustack[MAXARG], estack[MAX_ENV+1], stackbase;
//   printf("print exec stack...\n");
//   // argc
//   if(copyin(pagetable,(char*)&argc,sp,sizeof(uint64)) < 0){
//     printf("argc error\n");
//     return;
//   }
//   printf("[%p] argc:%d\n",sp,argc);

//   // argv vec
//   sp += sizeof(uint64);
//   if(copyin(pagetable,(char*)ustack,sp,(argc+1)*sizeof(uint64)) < 0){
//     printf("argv error\n");
//     return;
//   }

//   for(int i=0;i<argc;i++){
//     printf("[%p] argv[%d]:%p\n",sp+i*sizeof(uint64),i,ustack[i]);
//   }

//   // envp vec 

//   // AUX vec


// }

// Load a program segment into pagetable at virtual address va.
// va must be page-aligned
// and the pages from va to va+sz must already be mapped.
// Returns 0 on success, -1 on failure.
static int
loadseg(pagetable_t pagetable, uint64 va, struct ext4_file* f, uint offset, uint sz)
{
  uint i, n;
  int r = 0, rcnt = 0;
  uint64 pa;

 if((va % PGSIZE) != 0)
    panic("loadseg: va must be page aligned");
    
  for(i = 0; i < sz; i += PGSIZE){
    pa = walkaddr(pagetable, va + i);
    if(pa == 0)
      panic("loadseg: address should exist");
    if(sz - i < PGSIZE)
      n = sz - i;
    else
      n = PGSIZE;
    r = fread(f,(uint64)pa,offset+i,n,&rcnt,0);
    if( r!=EOK || rcnt != n)
      return -1;
  }
  
  return 0;
}

