#include "include/types.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/riscv.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/defs.h"
#include "include/kernel/kthread.h"

struct spinlock ktid_lock;
struct spinlock kthreadlist_lock;
int nextktid =1;
extern pagetable_t kernel_pagetable;
kthread kthreads[KTHREAD_NUM];
kthread *free_kthread=NULL;
kthread *alloc_kthread=NULL;

void kthreadinit(){
    initlock(&ktid_lock, "nextktid");
    initlock(&kthreadlist_lock,"kthreadlist");
    for(int i=0;i<KTHREAD_NUM;i++){
        initlock(&kthreads[i].lock, "kthread");
        kthreads[i].state=kt_UNUSED;
        kthreads[i].kpagetable=kernel_pagetable;
        kthreads[i].kstack=KSTACK(i);
        kthreads[i].prev=NULL;
        kthreads[i].next = NULL;
        kthreads[i].p=NULL;
        kthreads[i].t=NULL;
        if(i!=0){
            kthreads[i].prev=&kthreads[i-1];
        }
        if(i+1<KTHREAD_NUM){
            kthreads[i].next=&kthreads[i+1];
        }
    }
    free_kthread=&kthreads[0];
    kthreads[0].prev=NULL;
    kthreads[KTHREAD_NUM-1].next=NULL;
}

int allocktid()
{
    int ktid;

    acquire(&ktid_lock);
    ktid = nextktid++;
    release(&ktid_lock);

    return ktid;
}

kthread *allockthread()
{
    if (NULL == free_kthread)
    {
        panic("allockthread: can not find unused kthread");
    }
    acquire(&kthreadlist_lock);
    kthread *tmp = free_kthread;
    // acquire(&tmp->lock);
    free_kthread = free_kthread->next;
    tmp->next=NULL;
    free_kthread->prev=NULL;

    uint64 pa = (uint64)kalloc();

    if (pa == 0 || mappages(tmp->kpagetable, tmp->kstack, PGSIZE, (uint64)pa, PTE_R | PTE_W|PTE_A|PTE_D) < 0)
        panic("Allockthread: kstack mappages");
    tmp->kstack_pa=pa;
    tmp->ktid = allocktid();
    tmp->state = kt_USED;

    tmp->next = alloc_kthread;
    // uint64 a5=r_a5();
    // printf("a5: %p\t", a5);
    if(alloc_kthread)
        alloc_kthread->prev=tmp;
    alloc_kthread = tmp;
    alloc_kthread->prev=NULL;
    release(&kthreadlist_lock);
    
#ifdef debug
printf("%d ' kstack:%p\n",tmp->ktid,pa);
if(!tmp)panic("kthread NULL\n");
#endif

    // release(&tmp->lock);

    
    return tmp;
}

void createkthread(void(*func)(),char*name){
    kthread*kt=allockthread();
    
    safestrcpy(kt->name,name,sizeof(name));
    memset(&kt->context, 0, sizeof(kt->context));
    kt->context.ra=(uint64)func;
    kt->context.sp=kt->kstack+PGSIZE;
    acquire(&kt->lock);
    kt->state=kt_RUNNABLE;
    release(&kt->lock);

    
}

void freekthread(kthread*kt){
    kt->state=kt_UNUSED;
    return;
    uvmunmap(kt->kpagetable,kt->kstack,1,1);

    kt->p=NULL;
    kt->t=NULL;

    acquire(&kthreadlist_lock);
    if(kt==alloc_kthread){
        alloc_kthread=kt->next;
        if(kt->next)
            alloc_kthread->prev=NULL;
    }else{
        kt->prev->next=kt->next;
        if(kt->next)
            kt->next->prev=kt->prev;
    }
    kt->next=free_kthread;
    free_kthread->prev=kt;
    kt->prev=NULL;
    free_kthread=kt;
    release(&kthreadlist_lock);
}
