//
// File-system system calls.
// Mostly argument checking, since we don't trust
// user code, and calls into file.c and fs.c.
//

#include "include/types.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/kernel/param.h"
#include "include/fs/vfs/stat.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/fs/vfs/fs.h"
#include "include/kernel/atomic/sleeplock.h"
#include "include/fs/vfs/file.h"
#include "include/fs/vfs/fcntl.h"
// #include "include/fs/fat32/fat32.h"
#include "include/init/sbi.h"
#include "include/syscall/ioctl.h"
//  #define DEBUG
// Fetch the nth word-sized system call argument as a file descriptor
// and return both the descriptor and the corresponding struct file.
// sysfile里面只有前两个函数调用：argfd和fdalloc可以保留，其他的都需要再重写

uint64 getcwd(struct dirent *de, int isusr, uint64 buf, int max_len);

/*获取fd和其对应的file*/
static int
argfd(int n, int *pfd, struct file **pf)
{
  int fd;
  struct file *f;

  argint(n, &fd);
  if (fd < 0 || fd >= NOFILE || (f = myproc()->ofile[fd]) == 0)
    return -1;
  if (pfd)
    *pfd = fd;
  if (pf)
    *pf = f;
  return 0;
}

// Allocate a file descriptor for the given file.
// Takes over file reference from caller on success.
static int
fdalloc(struct file *f,int num)
{
  int fd;
  struct proc *p = myproc();
  for (fd = num; fd < NOFILE; fd++)
  {
    if (p->ofile[fd] == 0)
    {
      p->ofile[fd] = f;
      return fd;
    }
  }
  return -1;
}

uint64
sys_dup(void)
{
  struct file *f;
  int fd;

  if (argfd(0, 0, &f) < 0)
    return -1;
  if ((fd = fdalloc(f,0)) < 0)
    return -1;
  filedup(f);
  return fd;
}

uint64
sys_dup3(void)
{
  struct file *f;
  int oldfd;
  int newfd;
  struct proc *p = myproc();

  if (argfd(0, &oldfd, &f) < 0)
    return -1;
  argint(1, &newfd);
#ifdef DEBUG
  printf("[dup2] oldfd is %d\n", oldfd);
  printf("[dup2] newfd is %d\n", newfd);
#endif
  if (newfd < 0 || newfd > NOFILE) // new fd out of boundary
    return -1;
  if (p->ofile[newfd] == NULL)
  {
    p->ofile[newfd] = f;
    filedup(f);
    return newfd;
  }
  else
  { // new fd was occupied
    fileclose(p->ofile[newfd]);
    p->ofile[newfd] = f;
    filedup(f);
    return newfd;
  }
}

uint64
sys_read(void)
{
  struct file *f;
  int n,fd;
  uint64 p;

  argaddr(1, &p);
  argint(2, &n);
  if (argfd(0, &fd, &f) < 0)
    return -1;
  // printf("[sys_read] fd:%d addr:%p n:%d\n",fd,p,n);
  // pte_t *pte=walk(myproc()->pagetable,p,0);
  // uint64 pa=PTE2PA(*pte);
  return fileread(f, p, n);
}

uint64
sys_write(void)
{
  struct file *f;
  int n;
  uint64 p;

  argaddr(1, &p);
  argint(2, &n);
  if (argfd(0, 0, &f) < 0)
    return -1;

  return filewrite(f, p, n,1);
}

uint64
sys_close(void)
{
  int fd;
  struct file *f;

  if (argfd(0, &fd, &f) < 0)
    return -1;
  myproc()->ofile[fd] = 0;
  fileclose(f);
  return 0;
}

uint64
sys_fstat(void)
{
  // struct file *f;
  // uint64 addr; // user pointer to struct stat

  // argaddr(1, &addr);
  // if (argfd(0, 0, &f) < 0)
  //   return -1;
  
  // struct stat st;
  // struct ext4_inode_ref ref;
  // printf("11\n");
  // // 此处有问题，f->file 为 空指针
  // if(f->file == 0){
  //   f->file = (struct ext4_file *)kalloc();
  // }
  // ext4_file_get_inode_ref(&f->file, &ref);
  // printf("22\n");
  // st.st_dev = 0;
  // st.st_ino = ref.index;
  // st.st_mode = ref.inode->mode;
  // st.st_nlink = ref.inode->size_lo;
  // st.st_uid = 0;
  // st.st_gid = 0;
  // st.st_rdev = 0;
  // st.st_size = ref.inode->size_lo;
  // st.st_blksize = ref.inode->size_lo / ref.inode->blocks_count_lo;
  // st.st_blocks = (uint64)ref.inode->blocks_count_lo;
  // st.st_atime = 0;
  // st.st_atimensec = 0;
  // st.st_ctime = 0;
  // st.st_ctimensec = 0;
  // st.st_mtime = 0;
  // st.st_mtimensec = 0;

  // if(copyout(myproc()->pagetable,addr,(char*)&st,sizeof(st)) < 0){
  //   return -1;
  // }
  return 0;

}

// Create the path new as a link to the same inode as old.
uint64
sys_linkat(void)
{
//   int oldDirFd, oldaddr, newaddr, newDirFd, flags;
//   argint(0, &oldDirFd);
//   argint(1, &oldaddr);
//   argint(2, &newDirFd);
//   argint(3, &newaddr);
//   argint(4, &flags);

//   // assert(flags == 0);//todo include assert.h
//   if (flags != 0)
//     return -1;
//   char oldPath[EXT4_MAX_PATH], newPath[EXT4_MAX_PATH];
//   if (fetchstr((uint64)oldaddr, oldPath, EXT4_MAX_PATH) < 0)
//   {
//     // tf->a0 = -1;
//     return -1;
//   }
//   if (fetchstr((uint64)newaddr, newPath, EXT4_MAX_PATH) < 0)
//   {
//     // tf->a0 = -1;
//     return -1;
//   }

//   struct dirent *entryPoint, *targetPoint = NULL;
//   if ((entryPoint = ename(oldDirFd, oldPath)) == NULL)
//   {
//     goto bad;
//   }

//   if ((targetPoint = create(newDirFd, newPath, T_FILE, O_RDWR)) == NULL)
//   {
//     goto bad;
//   }

//   char buf[EXT4_MAX_PATH];
//   if (getcwd(entryPoint, 0, (u64)buf, EXT4_MAX_PATH) < 0)
//   {
//     goto bad;
//   }

//   int len = strlen(buf);
//   if (ewrite(targetPoint, 0, (u64)buf, 0, strlen(buf)) != len)
//   {
//     goto bad;
//   }

//   targetPoint->_nt_res = DT_LNK;
//   eupdate(targetPoint);
// bad:
//   return -1;

  return 0;
}

uint64
sys_unlinkat(void)
{
//   int dirFd, addr, flags;
//   argint(0, &dirFd);
//   argint(1, &addr);
//   argint(2, &flags);

//   // assert(flags == 0);//todo assert
//   if (flags != 0)
//     return -1;
//   char path[EXT4_MAX_PATH];
//   if (fetchstr((uint64)addr, path, EXT4_MAX_PATH) < 0)
//   {
//     // tf->a0 = -1;
//     return -1;
//   }
//   struct dirent *entryPoint;

//   if ((entryPoint = ename(dirFd, path)) == NULL)
//   {
//     goto bad;
//   }

//   entryPoint->_nt_res = 0;
//   eremove(entryPoint);

//   return 0;
//   return 0;
// bad:
//   // tf->a0 = -1;
  return -1;
}

uint64
sys_open(void)
{
  // char path[EXT4_MAX_PATH];
  // int fd, omode;
  // struct file *f;

  // if (argstr(0, path, EXT4_MAX_PATH) < 0)
  // {
  //   printf("[sys_open]parse para 0 error\n");
  //   printf("path is %s\n", path);
  // }
  // argint(1, &omode);

  // if (omode & O_CREAT)
  // {
  //   ep = create(-100, path, T_FILE, omode);
  //   if (ep == NULL)
  //   {
  //     return -1;
  //   }
  // }
  // else
  // {
  //   if ((ep = ename(-100, path)) == NULL)
  //   {
  //     return -1;
  //   }
  //   elock(ep);
  //   if ((ep->attribute & ATTR_DIRECTORY) && omode != O_RDONLY)
  //   {
  //     eunlock(ep);
  //     eput(ep);
  //     return -1;
  //   }
  // }

  // if ((f = filealloc()) == NULL || (fd = fdalloc(f,0)) < 0)
  // {
  //   if (f)
  //   {
  //     fileclose(f);
  //   }
  //   eunlock(ep);
  //   eput(ep);
  //   return -1;
  // }

  // if (!(ep->attribute & ATTR_DIRECTORY) && (omode & O_TRUNC))
  // {
  //   etrunc(ep);
  // }

  // f->type = FD_ENTRY;
  // f->off = (omode & O_APPEND) ? ep->file_size : 0;
  // f->ep = ep;
  // f->readable = !(omode & O_WRONLY);
  // f->writable = (omode & O_WRONLY) || (omode & O_RDWR);

  // eunlock(ep);

  // return fd;
return 0;
}

// open :  syscall(SYS_openat, AT_FDCWD, path, flags, O_RDWR);
// openat: syscall(SYS_openat, dirfd, path, flags, 0600);
uint64
sys_openat(void)
{
  char path[EXT4_MAX_PATH], *file_path = NULL;
  int fd, flags, mode, r = 0, is_dir = 0;
  struct file *f;
  struct ext4_file file;
  struct ext4_dir dir;
  argint(0, &fd);
  if (argstr(1, path, EXT4_MAX_PATH) < 0)
  { 
    #ifdef debug
    printf("[sys_openat]parse para 0 error\n");
    printf("path is %s\n", path);
    #endif
    return -1;
  }
  argint(2, &flags);
  argint(3, &mode);
#ifdef DEBUG
  printf("[sys_openat] [0] fd is %d [1] path is %s [2] flags is %d [3] mode is %d\n", fd, path, flags, mode);
#endif

  if(!strcmp(path,"/etc/localtime") || !strcmp(path,"/proc/mounts") || !strcmp(path,"/proc") 
  || !strcmp(path,"/proc/meminfo") || !strcmp(path,"/dev/misc/rtc") || !strcmp(path,"/etc/adjtime") || !strcmp(path,"/dev/rtc") ){
    if ((f = filealloc()) == NULL || (fd = fdalloc(f, 0)) < 0){
      if (f)
        fileclose(f);
      return -1;
    }

    f->is_dir = 0;
    f->type = FD_DUMMY;
    f->off = 0;
    f->readable = !(flags & O_WRONLY);
    f->writable = (flags & O_WRONLY) || (flags & O_RDWR);
    return fd;
  }

  if(is_relative(path)){
    if(argfd(0, &fd, &f) < 0){
      if(fd != AT_FDCWD)
        return -1;
      file_path = to_abspath(path);
    }
    else{
      printf("[sys_openat] relative dirfd not implemented!\n");
    }
  }
  else{
    file_path = path;
  }

  if((r = fopen(&file, file_path, flags)) != EOK){
    if((r = dir_open(&dir, file_path)) != EOK)
      return -1;
    is_dir = 1;
  }

  int fd_new;
  if ((f = filealloc()) == NULL || (fd_new = fdalloc(f,0)) < 0)
  {
    if (f)
    {
      fileclose(f);
    }
    return -1;
  }

  if(is_dir)
    f->dir = dir;    
  else
    f->file = file;
  f->is_dir = is_dir;
  f->type = FD_ENTRY;
  f->off = (flags & F_APPEND) ? f->file.fsize : 0;
  f->readable = !(flags & F_WRONLY);
  f->writable = (flags & F_WRONLY) || (flags & F_RDWR);
  return fd_new;
}

uint64
sys_mkdirat(void)
{
  char path[EXT4_MAX_PATH];
  int dirFd, addr, mode;
  argint(0, &dirFd);
  argint(1, &addr);
  argint(2, &mode);

#ifdef DEBUG
  printf("[mkdir] dirfd is %d\n", dirFd);
  printf("[mkdir] addr is 0x%x\n", addr);
  printf("[mkdir] mode is %d\n", mode);
#endif
  if (fetchstr((uint64)addr, path, EXT4_MAX_PATH) < 0)
  {
    // tf->a0 = -1;
    return -1;
  }
  else
  {
#ifdef DEBUG
    printf("[mkdir] path is %s\n", path);
#endif
  }
  printf("[mkdir] path is %s\n", path);
  int r = mkdir(path);
  if(r == EOK){
    return 0;
  }else{
    goto bad;
  }

bad:
#ifdef DEBUG
  printf("[mkdir] create dir error\n");
#endif
  return -1;


}

// To open console device.
uint64
sys_mknod(void)
{
  int fd, omode;
  int major, minor;
  struct file *f;

  argint(0, &fd);
  argint(1, &omode);
  argint(2, &major);
  argint(3, &minor);

  if (omode & O_CREAT)
  {
    panic("dev file on FAT");
  }

  if (major < 0 || major >= NDEV)
    return -1;

  if ((f = filealloc()) == NULL || (fd = fdalloc(f,0)) < 0)
  {
    if (f)
      fileclose(f);
    return -1;
  }

  f->type = FD_DEVICE;
  f->off = 0;
  f->major = major;
  f->readable = !(omode & O_WRONLY);
  f->writable = (omode & O_WRONLY) || (omode & O_RDWR);

  return fd;
}

uint64
sys_chdir(void)
{
  char path[EXT4_MAX_PATH];
  struct ext4_dir d;
  struct proc *p = myproc();
  int r;

  if (argstr(0, path, EXT4_MAX_PATH) < 0 || ( r = dir_open(&d,path)) != EOK)
  {
#ifdef DEBUG
    printf("[ERROR] path is %s\n", path);
#endif
    return -1;
  }

  p->cwd = d;
  mykthread()->p->cwd = d;
#ifdef DEBUG
  printf("[sys_chdir] chdir to %s\n",  p->cwd->de.name;)
#endif
  return 0;
}

// uint64
// sys_exec(void)
// {
//   char path[MAXPATH], *argv[MAXARG], *envp[] = {0};
//   int i;
//   uint64 uargv, uarg;
//   argaddr(1, &uargv);
//   if (argstr(0, path, MAXPATH) < 0)
//   {
//     return -1;
//   }
//   memset(argv, 0, sizeof(argv));
//   for (i = 0;; i++)
//   {
//     if (i >= NELEM(argv))
//     {
//       goto bad;
//     }
//     if (fetchaddr(uargv + sizeof(uint64) * i, (uint64 *)&uarg) < 0)
//     {
//       goto bad;
//     }
//     if (uarg == 0)
//     {
//       argv[i] = 0;
//       break;
//     }
//     argv[i] = kalloc(); // 每次都分配一页，浪费内存
//     if (argv[i] == 0)
//       goto bad;
//     if (fetchstr(uarg, argv[i], PGSIZE) < 0)
//       goto bad;
//   }

//   int ret = exec(path, argv, envp);

//   for (i = 0; i < NELEM(argv) && argv[i] != 0; i++)
//     kfree(argv[i]);

//   return ret;

// bad:
//   for (i = 0; i < NELEM(argv) && argv[i] != 0; i++)
//     kfree(argv[i]);
//   return -1;
// }

uint64
sys_execve(void)
{
  char path[MAXPATH], *argv[MAXARG], *envp[MAXARG];
  int i;
  uint64 uargv, uarg, uenvp, uenv;

  argaddr(1, &uargv);
  argaddr(2, &uenvp);
  if (argstr(0, path, MAXPATH) < 0)
  {
    return -1;
  }
  memset(argv, 0, sizeof(argv));
  for (i = 0;; i++)
  {
    if (i >= NELEM(argv))
    {
      goto bad;
    }
    if (fetchaddr(uargv + sizeof(uint64) * i, (uint64 *)&uarg) < 0)
    {
      goto bad;
    }
    if (uarg == 0)
    {
      argv[i] = 0;
      break;
    }
    argv[i] = kalloc(); // 每次都分配一页，浪费内存
    if (argv[i] == 0)
      goto bad;
    if (fetchstr(uarg, argv[i], PGSIZE) < 0)
      goto bad;
  }

  memset(envp, 0, sizeof(envp));
  for (i = 0;; i++)
  {
    if (i >= NELEM(envp))
    {
      goto bad;
    }
    if (fetchaddr(uenvp + sizeof(uint64) * i, (uint64 *)&uenv) < 0)
    {
      goto bad;
    }
    if (uenv == 0)
    {
      envp[i] = 0;
      break;
    }
    envp[i] = kalloc(); // 每次都分配一页，浪费内存
    if (envp[i] == 0)
      goto bad;
    if (fetchstr(uenv, envp[i], PGSIZE) < 0)
    {
      printf("here %p\n", uenv);
      goto bad;
    }
  }

  int ret = execve(path, argv, envp);

  for (i = 0; i < NELEM(argv) && argv[i] != 0; i++)
    kfree(argv[i]);
  for (i = 0; i < NELEM(envp) && envp[i] != 0; i++)
    kfree(envp[i]);

  return 0;

bad:
  for (i = 0; i < NELEM(argv) && argv[i] != 0; i++)
    kfree(argv[i]);
  return -1;
}

uint64
sys_pipe2(void)
{
  uint64 fdarray; // user pointer to array of two integers
  struct file *rf, *wf;
  int fd0, fd1;
  struct proc *p = myproc();

  argaddr(0, &fdarray);
  if (pipealloc(&rf, &wf) < 0)
    return -1;
  fd0 = -1;
  if ((fd0 = fdalloc(rf,0)) < 0 || (fd1 = fdalloc(wf,0)) < 0)
  {
    if (fd0 >= 0)
      p->ofile[fd0] = 0;
    fileclose(rf);
    fileclose(wf);
    return -1;
  }
  if (copyout(p->pagetable, fdarray, (char *)&fd0, sizeof(fd0)) < 0 ||
      copyout(p->pagetable, fdarray + sizeof(fd0), (char *)&fd1, sizeof(fd1)) < 0)
  {
    p->ofile[fd0] = 0;
    p->ofile[fd1] = 0;
    fileclose(rf);
    fileclose(wf);
    return -1;
  }
  printf("[sys_pipe2] fd0:%d fd1:%d\n",fd0,fd1);
  return 0;
}

// To open console device.
uint64
sys_dev(void)
{
  int fd, omode;
  int major, minor;
  struct file *f;

  // if(argint(0, &omode) < 0 || argint(1, &major) < 0 || argint(2, &minor) < 0){
  //   return -1;
  // }
  argint(0, &omode);
  argint(1, &major);
  argint(2, &minor);

  if (omode & O_CREAT)
  {
    panic("dev file on FAT");
  }

  if (major < 0 || major >= NDEV)
    return -1;

  if ((f = filealloc()) == NULL || (fd = fdalloc(f,0)) < 0)
  {
    if (f)
      fileclose(f);
    return -1;
  }

  f->type = FD_DEVICE;
  f->off = 0;
  // f->file = 0;
  f->major = major;
  f->readable = !(omode & O_WRONLY);
  f->writable = (omode & O_WRONLY) || (omode & O_RDWR);

  return fd;
}

uint64
getcwd(struct dirent *de, int isusr, uint64 buf, int max_len)
{

  // if (de->parent == NULL)
  // {
  //   // s-=1;
  //   // *s = "/";
  //   return either_copyout(isusr, buf, "/", 2);
  // }

  // char path[FAT32_MAX_FILENAME];
  // // int len;
  // // strncpy(path, '\0', max_len);
  // u8 *s = (u8 *)path + EXT4_MAX_PATH - 1;
  // *s = '\0';
  // int len;
  // while (de->parent)
  // {
  //   len = strlen(de->filename);
  //   s -= len;
  //   if (s <= (u8 *)path || (s - (u8 *)path) <= (FAT32_MAX_FILENAME - max_len))
  //     return -1;
  //   strncpy((char *)s, de->filename, len);
  //   *--s = '/';
  //   de = de->parent;
  // }
  // return either_copyout(isusr, buf, (void *)s, strlen((char *)s) + 1);
  return 0;
}

// syscall(SYS_getcwd, buf, size);
uint64
sys_getcwd(void)
{
  // uint64 addr;
  // int size;

  // argaddr(0, &addr);
  // if (addr == NULL)
  // { // buf 为NULL，系统分配缓存区
  //   printf("[sys_getcwd] buf is NULL\n");
  //   addr = (uint64)kalloc();
  //   mappages(myproc()->pagetable, addr, PGSIZE, addr, PTE_R | PTE_W);
  // }
  // argint(1, &size);

  // struct ext4_dir* d = &mykthread()->p->cwd;

  // if(copyout(myproc()->pagetable,addr,(char*)d->de.name,d->de.name_length+1) < 0){
  //   panic("getcwd error\n");
  // }
  // return addr;

  
  uint64 addr;
  argaddr(0, &addr);

  char *mp = "/";

  if (copyout(myproc()->pagetable, addr, mp, strlen(mp) + 1) < 0)
    return -1;
  // printf("[sys_getcwd] cwd: %s, cwd_len: %d, addr: %p\n", mp, strlen(mp) + 1, addr);
  return addr;


}

int
direntnext(struct file *f, uint64 buf, int len)
{
  #ifdef debug  
    printf("[direntnext] buf: %p, len: %d\n", buf, len);
  #endif

  if(f->readable == 0)
    return -1;

  uint64 nread = len;

  // de64 虚拟地址
  // struct linux_dirent64* de64 = (struct linux_dirent64*)buf;
  struct linux_dirent64* de64 = (struct linux_dirent64*)kalloc();
  struct ext4_dir *dir = &f->dir;
  const struct ext4_direntry *dentry = NULL;
  
  if(f->type == FD_ENTRY) {
    while ((dentry = ext4_dir_entry_next(dir))) {
		  if (!dentry->inode) 
			  continue;

      de64->d_ino = (uint64)dentry->inode;
      de64->d_off = dir->next_off;
      de64->d_type = dentry->inode_type;
      memmove((uint64)de64->d_name, (char*)dentry->name, dentry->name_length + 1);
      int size = ((uint64)de64->d_name - (uint64)de64) + dentry->name_length + 1;
      size += (sizeof(uint64) - (size % sizeof(uint64))) % sizeof(uint64);
      de64->d_reclen = size;

      // printf("[direntnext] de64: %p, size: %d, de64name: %s, length: %d\n", (uint64)de64, size, (char*)de64->d_name, dentry->name_length);
      // printf("[direntnext] off: %p, ino: %p, type: %d, length: %d\n", dir->next_off, dentry->inode, dentry->inode_type, dir->de.entry_length);

      // buff overflow
      if(size > len) {
        // printf("[direntnext] buff overflow\n");
        break;
      }

      if(copyout(myproc()->pagetable, buf, (char*)de64, size) < 0) {
        kfree(PGROUNDDOWN((uint64)de64));
        return -1;
      }
      buf+=size;
      de64 = (struct linux_dirent64*)((uint64)de64 + size);
      len -= size;
	  }
	  // ext4_dir_entry_rewind(dir);
  }
  kfree(PGROUNDDOWN((uint64)de64));
  return nread - len;
}

uint64
sys_getdents64(void)
{
  int fd, len;
  struct file* f;
  uint64 buf;

  if (argfd(0, &fd, &f) < 0)
    return -1;
  argaddr(1, &buf);
  argint(2, &len);
  if(f->type == 9)
    return 0;

  return direntnext(f, buf, len);
}

uint64
sys_mount(void)
{
  // uint64 imagePathva, mountPathva, typeva, datava;
  // int flags;
  // char imagePath[FAT32_MAX_FILENAME], mountPath[FAT32_MAX_FILENAME], type[10], data[10];
  // argaddr(0, &imagePathva);
  // argaddr(1, &mountPathva);
  // argaddr(2, &typeva);
  // argint(3, &flags);
  // argaddr(0, &datava);
  // if (fetchstr(typeva, type, 10) < 0 || strncmp(type, "vfat", 4))
  // {
  //   return -1;
  // }
  // struct dirent *ep, *dp;
  // if (fetchstr(imagePathva, imagePath, EXT4_MAX_PATH) < 0 || (ep = ename(AT_FDCWD, imagePath)) == NULL)
  // {
  //   return -1;
  // }
  // // printf("reach 2222\n");
  // if (fetchstr(mountPathva, mountPath, EXT4_MAX_PATH) < 0 || (dp = ename(AT_FDCWD, mountPath)) == NULL)
  // {
  //   return -1;
  // }
  // if (datava && fetchstr(datava, data, 10) < 0)
  // {
  //   return -1;
  // }
  // FileSystem *fs;
  // if (fsAlloc(&fs) < 0)
  // {
  //   return -1;
  // }

  // struct file *file = filealloc();
  // file->off = 0;
  // file->readable = true;
  // file->writable = true;
  // if (ep->head)
  // {
  //   file->type = ep->head->image->type;
  //   file->ep = ep->head->image->ep;
  // }
  // else
  // {
  //   file->type = FD_ENTRY;
  //   file->ep = ep;
  // }
  // fs->name[0] = 'm';
  // fs->name[1] = 0;
  // fs->image = file;
  // fs->read = mountBlockRead;
  // if (file->type != FD_ENTRY)
  //   fatInit(fs);
  // else
  // {
  //   fs->root = *ep;
  // }
  // fs->next = dp->head;
  // dp->head = fs;
  return 0;
}

uint64
sys_umount2(void)
{
  // uint64 mountPathva;
  // int flags;
  // char mountPath[FAT32_MAX_FILENAME];
  // struct dirent *ep;
  // argaddr(0, &mountPathva);
  // argint(1, &flags);
  // if (fetchstr(mountPathva, mountPath, EXT4_MAX_PATH) < 0 || (ep = ename(AT_FDCWD, mountPath)) == NULL)
  // {
  //   return -1;
  // }

  // if (ep->head == NULL)
  // {
  //   return -1;
  // }

  // ep->head->valid = 0;
  // ep->head = ep->head->next;

  return 0;
}

uint64
sys_shutdown(void)
{
  int sysfail;
  argint(0, &sysfail);
  sbi_shutdown(sysfail);
  return 0;
}

// 功能：检查进程实际用户ID和实际组ID对文件的访问权限；
// 输入：
// - dirfd：文件所在目录的文件描述符。
// - pathname：文件路径
// - mode: 测试权限，如 F_OK、R_OK、W_OK、X_OK
// - flags:
// 返回值：成功执行，返回0。错误，则返回-1。
uint64 sys_faccessat(void)
{
  int dirfd, mode, flags;
  uint32 imode, ino;
  char pathname[EXT4_MAX_PATH];
  struct ext4_inode ext_inode;
  argint(0, &dirfd);
  argint(2, &mode);
  argint(3, &flags);
  if (argstr(1, pathname, EXT4_MAX_PATH) < 0)
  {
    return -1;
  }

  if(is_relative(pathname)){
    if(dirfd != AT_FDCWD)
      return -1;
    return -1;
  }
  
  int r;
  if((r = ext4_raw_inode_fill(pathname, &ino, &ext_inode)) != EOK)
    return -1;

  printf("[sys_faccessat] mode: %p\n",ext_inode.mode);
  if (mode == F_OK)
    return 0;

  imode = (ext_inode.mode >> 6) & 0x7;
  if ((imode & mode) != mode)
    return -1;

  return 0;
}



// 功能：改变已经打开文件的属性
// 输入：
// - fd：文件所在目录的文件描述符。
// - cmd：功能描述
// - arglist:
// 返回值：成功执行，返回值依赖于cmd。错误，则返回-1。
uint64 sys_fcntl(void)
{
int fd, cmd, arg, ret;
  struct file *f;
  argint(1, &cmd);
  argint(2, &arg);
  if (argfd(0, 0, &f) < 0 )
    return -1;
  #ifdef debug
    printf("[sys_fcntl] cmd: %d, f: %p, arg: %d\n", cmd, f, arg);
  #endif
  switch (cmd) {
    case F_DUPFD:
      if((fd=fdalloc(f, arg)) < 0)
        return -1;
      filedup(f);
      fd_set_exec_flag(f, fd, 0);
      ret = fd;
      break;
    case F_DUPFD_CLOEXEC:
      argint(0, &fd);
      int flag = fd_get_exec_flag(f, fd);
      // printf("[sys_fcntl] fd: %d, flag: %d\n", fd, flag);
      if((fd=fdalloc(f, arg)) < 0)
        return -1;
      filedup(f);
      fd_set_exec_flag(f, fd, flag);
      ret = fd;
      break;
    case F_GETFD:
      argint(0, &fd);
      ret = fd_get_exec_flag(f, fd) ? FD_CLOEXEC : 0;
      break;
    case F_SETFD:
      argint(0, &fd);
      // printf("[sys_fcntl] fd: %d\n", fd);
      fd_set_exec_flag(f, fd, arg & FD_CLOEXEC);
      ret = 0;
      break;
    case F_GETFL:
      ret = f->file.flags;
      break;
    case F_SETFL:
      f->file.flags = arg;
      ret = 0;
      break;
    default:
      printf("[sys_fcntl] unsupported cmd: %d!\n", cmd);
      return -1;
  }

  return ret;

}


// uint64 sys_sendfile(void) {
//   int out_fd;
//   int in_fd;
//   struct file *fout;
//   struct file *fin;
//   uint64 offset;
//   uint64 count;
//  argfd(0, &out_fd, &fout);
//  argfd(1, &in_fd, &fin);
//   argaddr(2, &offset);
//   argaddr(3, &count);
//   // debug_print("pid: %d out_type:%d in_type:%d\n", myproc()->pid, fout->type,
//   //             fin->type);
//   // if (fin->type == FD_ENTRY) {
//   //   debug_print("in name : %s\n", fin->ep->filename);
//   // }
//   return file_send(fin, fout, offset, count);
// }

uint64 sys_ioctl(void){
  int fd,cmd;
  struct file * f;
  uint64 va;
  struct winsize ws;
  if(argfd(0,&fd,&f)<0){
    #ifdef debug
    printf("[sys_ioctl] get fd error\n");
    #endif
  }
  argint(1,&cmd);
  argaddr(2,&va);
  #ifdef debug
  printf("[sys_ioctl] fd is %d | cmd is %p\n",fd,cmd);
  #endif
  switch(cmd){
    case TIOCGWINSZ:{
      if(copyin(myproc()->pagetable,(char*)&ws,va,sizeof(struct winsize)) < 0){
        #ifdef debug
        printf("[sys_ioctl] get winsize error\n");
        #endif
        return -1;
      }
      // printf("%d %d\n",ws.ws_col,ws.ws_row);
    }
      break;
    default:
      return 0;
  }

  return 0;
}

uint64 sys_writev(void){
  int fd, iovcnt;
  struct file* f;
  uint64 iov_arr;
  if(argfd(0,&fd,&f)<0){
    #ifdef debug
    printf("[sys_writev] get fd error\n");
    #endif
    return -1;
  }
  argaddr(1,&iov_arr);
  argint(2,&iovcnt);
  

  // printf("[sys_writev] fd:%d | iov_arr:%p | iovcnt:%d\n",fd,iov_arr,iovcnt);

  struct iovec *iov = (struct iovec*)kmalloc(iovcnt * sizeof(struct iovec));
  if(iov == NULL)
    return -1;

  if(copyin(myproc()->pagetable,(char*)iov,iov_arr,sizeof(struct iovec)*iovcnt) < 0){
    return -1;
  }
  // for(int i=0;i<iovcnt;i++){
  //   printf("iov[%d] iov_base:%p, iov_len:%p\n",i,iov[i].iov_base,iov[i].iov_len);
  // }

  int ret;
  ret = filewritev(f, iov, iovcnt);

  kmfree(iov);
  return ret;
}

uint64 sys_readv(void){
  return 0;
}


// unsigned int fd, unsigned long offset_high, unsigned long offset_low, loff_t __user *result, unsigned int whence
uint64 sys_lseek(void){
  int fd, whence;
  uint64 offset;
  struct file* f;
  if(argfd(0,&fd,&f)<0 ){
    return -1;
  }
  argaddr(1,&offset);
  argint(2,&whence);
  
  if(f->type == FD_ENTRY){
    switch (whence)
    {
    case SEEK_CUR: // The file offset is set to its current location plus offset bytes.
      f->off += offset;
      break;
    case SEEK_SET: // The file offset is set to offset bytes.
      f->off=offset;
      break; 
    case SEEK_END: // The file offset is set to the size of the file plus offset bytes.
      f->off = f->file.fsize + offset;
      break;
    default:
      break;
    }
  }

  return f->off;
}

// int olddfd, const char __user *oldname, int newdfd, const char __user *newname, unsigned int flags
uint64 sys_renameat2(void){
  // int oldfd,newfd;
  return 0;
}

// int out_fd, int in_fd, loff_t __user *offset, size_t count
/* copies data between one file descriptor and another 
   in_fd open for reading, out_fd open for writing
*/

uint64 sys_sendfile(void){
  int out_fd, in_fd, cnt;
  struct file *out_f, *in_f;
  uint64 offset; // TODO: support offset
  if( argfd(0,&out_fd,&out_f) < 0 || argfd(1,&in_fd,&in_f) < 0){
    return -1;
  }
  argaddr(2,&offset);
  argint(3,&cnt);

  int ret = 0;
  char *buf = (char*)kalloc();
  for(int i = 0; i < cnt; i += PGSIZE){
    int n;
    if(cnt - i > PGSIZE)
      n = PGSIZE;
    else
      n = cnt - 1;
    int rcnt;
    rcnt = fileread(in_f, (uint64)buf, n);
    ret += filewrite(out_f, (uint64)buf, rcnt,0);
    if(rcnt != n)
      break;
  }

  kfree(buf);

  printf("[sys_sendfile] ret: %d\n", ret);
  return ret;

}

// const char __user *path, size_t sz, struct statfs64 __user *buf
/*
  The statfs() system call returns information about a mounted filesystem. 
  path is the pathname of any file within the mounted filesystem.
  buf is a pointer to a statfs structure defined approximately as follows:
*/
uint64 sys_statfs(void){
  char path[EXT4_MAX_PATH];
  uint64 buf;

  if( argstr(0,path,EXT4_MAX_PATH) < 0 ){
    return -1;
  }
  argaddr(1,&buf); 

  struct ext4_mount_stats stats;
  ext4_mount_point_stats("/", &stats);

  struct statfs st;
  st.f_type = EXT4_SUPERBLOCK_MAGIC;
  st.f_bsize = stats.block_size;
  st.f_blocks = stats.blocks_count;
  st.f_bfree = stats.free_blocks_count;
  st.f_bavail = stats.free_blocks_count;
  st.f_files = stats.inodes_count;
  st.f_ffree = stats.free_inodes_count;
  st.f_fsid.val[0] = 0;
  st.f_fsid.val[1] = 0;
  st.f_namelen = 16;
  st.f_frsize = 0;
  st.f_flags = 0;

  if(copyout(myproc()->pagetable,buf,(char*)&st,sizeof(struct statfs))<0) {
    return -1;
  }
  return 0;
}

uint64 sys_sysinfo(void){
  return 0;
}

uint64 sys_syslog(void){
  return 0;
}

uint64 sys_utimensat(void){
  return 0;
}

// int dfd, const char __user *filename, struct stat __user *statbuf, int flag
uint64 sys_fstatat(void){
  int fd,flags;
  char path[EXT4_MAX_PATH];
  uint64 buf;
  struct file *f;
  argint(0,&fd);
  if(argstr(1,path,EXT4_MAX_PATH) < 0 ){
    return -1;
  }
  argaddr(2,&buf);
  argint(3,&flags);
  #ifdef debug 
    printf("[sys_fstatat] dirfd: %d, path: %s, statbuf: %p, flags: %d\n", fd, path, buf, flags);
  #endif

  struct stat st;
    
  if(fkstatat(path, &st, fd) != 0)
    return -1;
  if(copyout(myproc()->pagetable, buf, (char*)&st, sizeof(st)) < 0)
    return -1;

  return 0;
}

uint64 sys_ppoll(void){
  int nfds;
  uint64 fdsaddr, tmoaddr;
	argaddr(0, &fdsaddr); 
  argint(1, (int*)&nfds);
  argaddr(2, &tmoaddr);

  // printf("[sys_ppoll] fdsaddr: %p, nfds: %d, tmoaddr: %p\n", fdsaddr, nfds, tmoaddr);

	// struct pollfd fds;

	// for (int i = 0; i < nfds; i++) {
	// 	if (copyin2((char*)((uint64)&fds + i), fdsaddr + i * sizeof(struct pollfd), sizeof(struct pollfd)) < 0)
	// 		return -1;
	// }

  // printf("[sys_ppoll] fds->fd: %d\n", fds->fd);

  return nfds;
}

uint64 sys_readlinkat(void) {
  int bufsiz;
  uint64 addr2;
  struct file f;
  char path[EXT4_MAX_PATH];
  int dirfd = 0;
  char buf[1024];
  int pathlen;
  struct proc *p = myproc();

  argfd(0, &dirfd, &f);
  argstr(1, path, EXT4_MAX_PATH);
  argaddr(2, &addr2);
  argint(3, &bufsiz);
  
  if(dirfd==AT_FDCWD){
    pathlen= strlen(p->name)+1;
    copyout(p->pagetable,addr2,"/",1);
    copyout(p->pagetable,addr2+1,p->name,pathlen-1);
  }else{
    pathlen= strlen(path);
    copyout(p->pagetable, addr2, path,pathlen);
  }
  return pathlen;
}