#include "include/types.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/kernel/param.h"
#include "include/fs/vfs/stat.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/fs/vfs/fs.h"
#include "include/kernel/atomic/sleeplock.h"
#include "include/fs/vfs/file.h"
#include "include/fs/vfs/fcntl.h"
// #include "include/fs/fat32/fat32.h"

uint64 sys_mmap(void)
{
  uint64 addr;
  int len, prot, flags, fd, off;
  argaddr(0, &addr);
  argint(1, &len);
  argint(2, &prot);
  argint(3, &flags);
  argint(4, &fd);
  argint(5, &off);

  return mmap(addr, len, prot, flags, fd, off);
}

uint64 sys_munmap(void)
{
  // to gain the arguments and proc
  uint64 addr;
  int len;
  argaddr(0, &addr);
  argint(1, &len);

  return munmap(addr, len);
}

uint64
sys_sbrk(void)
{
  uint64 addr;
  int n;

  argint(0, &n);
  addr = myproc()->sz;
  if (growproc(n) < 0)
    return -1;
  return addr;
}

uint64
sys_brk(void)
{
  uint64 addr;
  uint64 n;

  argaddr(0, &n);
  // printf("[sys_brk] new_addr: %p, old_addr: %p\n", n, myproc()->sz);

  addr = myproc()->sz;
  if (n == 0) {
    return addr;
  }
  if (n >= addr) {
    if (growproc(n - addr) < 0)
      return -1;
    else
      return myproc()->sz;
  }
  return n;
}

int setperm(pagetable_t pagetable, uint64 va, uint64 perm) {
  pte_t *pte;

  if (va >= MAXVA)
    return -1;

  pte = walk(pagetable, va, 0);
  if (pte == 0||(*pte & PTE_V) == 0)
    return 0;
  if ((*pte & PTE_U) == 0)
    return -1;
  *pte |= perm;
  return 0;
}

uint64 sys_mprotect()
{
  uint64 addr, len;
  int prot;
  argaddr(0, &addr);
  argaddr(1, &len);
  argint(2, &prot);
  struct proc *p = myproc();
  struct VMA *vp = 0;
  //如果是mmap的地址，只修改
  for (int i = 0; i < VMA_MAX; i++)
    if (p->vma[i].addr <= addr && addr < p->vma[i].addr + p->vma[i].len && p->vma[i].valid == 1)
    {
      vp = &p->vma[i];
      vp->prot=prot;
      break;
    }
  
  int perm = PTE_U | (prot << 1);
  int page_n = PGROUNDUP(len) >> PGSHIFT;
  uint64 va = addr;
  for (int i = 0; i < page_n; i++)
  {
    if(setperm(p->pagetable, va, perm)==-1)
      panic("out of maxva or not user page"); 
    va += PGSIZE;
  }

  return 0;
}
