#include "include/types.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/kernel/param.h"
#include "include/fs/vfs/stat.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/fs/vfs/fs.h"
#include "include/kernel/atomic/sleeplock.h"
#include "include/fs/vfs/file.h"
#include "include/fs/vfs/fcntl.h"
// #include "include/fs/fat32/fat32.h"
#include "include/kernel/timer.h"
#include "include/kernel/uname.h"

extern struct proc proc[NPROC];

uint64
sys_sched_yield(void)
{
    yield();
  return 0;
}

uint64
sys_gettimeofday(void)
{
  uint64 addr;
  argaddr(0, &addr);
  uint64 t = r_time();
  struct timeval ts;
  ts.tv_sec = t / CLK_FREQ;
  ts.tv_usec = (t % CLK_FREQ) * 1000000 / CLK_FREQ;
  //printf("%d  %d\n",ts.tv_sec,ts.tv_usec);
  if(copyout(myproc()->pagetable, addr, (char *)&ts, sizeof(struct timeval))<0)
    return -1;
  return 0;
}

uint64
sys_uname(void)
{
  uint64 addr;
  argaddr(0, &addr);
  uname();
  if(copyout(myproc()->pagetable,addr,(char*)&uts,sizeof(uts))<0)
    return -1;
  return 0;
}

uint64
sys_times(void){
  struct tms ptms;
  uint64 utms;
  argaddr(0, &utms);
  struct proc *curr_p=myproc();
  ptms.tms_utime = curr_p->utime;
  ptms.tms_stime = curr_p->ktime;
  ptms.tms_cstime = 0;
  ptms.tms_cutime = 0;
  struct proc *p;
  for (p = proc; p < proc + NPROC; p++) {
    acquire(&p->lock);
    if (p->parent ==curr_p ) {
      ptms.tms_cutime += p->utime;
      ptms.tms_cstime += p->ktime;
    }
    release(&p->lock);
  }
  if(copyout(curr_p->pagetable, utms, (char *)&ptms, sizeof(ptms))<0)
    return -1;
  return ptms.tms_utime+ptms.tms_stime;
}

uint64
sys_nanosleep(void){
  int n;
  uint ticks0;
  uint64 addr0,addr1;
  struct proc*p=myproc();
  argaddr(0,&addr0);
  argaddr(1,&addr1);
  struct timespec req,rem;
  if(copyin(p->pagetable,(char*)&req,addr0,sizeof(struct timespec))<0)
    return -1;
  if(copyin(p->pagetable,(char*)&rem,addr1,sizeof(struct timespec))<0)
    return -1;
  n=req.tv_sec*ticks_per_second+req.tv_nsec*ticks_per_second/1000000000;
  acquire(&tickslock);
  ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(killed(p)){
      rem.tv_sec=(ticks-ticks0)/ticks_per_second;
      rem.tv_nsec=((ticks-ticks0)%ticks_per_second)*1000000000/ticks_per_second;
      copyout(p->pagetable,addr1,(char*)&rem,sizeof(rem));
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

uint64
sys_clock_gettime(void){
    int clockid;
    uint64 tp;
    struct timespec ts_buf;
    argint(0, &clockid);
    argaddr(1, &tp);
    uint64 t = r_time();
    ts_buf.tv_sec= t / CLK_FREQ;
    ts_buf.tv_nsec=(t % CLK_FREQ) * 1000000000 / CLK_FREQ;

    if (copyout(myproc()->pagetable, tp, (char *)&ts_buf, sizeof(ts_buf)) < 0) {
        return -1;
    }
  return 0;
}

uint64 sys_prlimit64() {
  struct proc*p=myproc();
  int pid;
  int resource;
  uint64 oldlimit;
  uint64 newlimit;
  rlimit r;
  rlimit oldr;
  oldr.rlim_cur=p->filelimit;
  argint(0, &pid);
  argint(1, &resource);
  argaddr(2, &newlimit);
  argaddr(3, &oldlimit);
  if (newlimit)
    if (copyin(p->pagetable, (void *)&r, newlimit, sizeof(rlimit)) < 0)
    {
      return -1;
    }
  if(r.rlim_cur>NOFILE)return -1;
  if (resource == 7) {
    myproc()->filelimit = r.rlim_cur;
  }
  if (copyout(p->pagetable, oldlimit,(void*)&oldr, sizeof(rlimit)) < 0) {
    return -1;
  }

  return 0;
}

