#include "include/types.h"
#include "include/riscv.h"
#include "include/defs.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/kernel/proc.h"
#include "include/fs/vfs/fcntl.h"
#include "include/kernel/futex.h"
// extern struct spinlock ulocks[NULOCK];



uint64
sys_exit(void)
{
  int n;
  argint(0, &n);
  printf("[sys_exit] %d\n",n);
  exit(n);
  return 0;  // not reached
}

// uint64
// sys_comp_exit(void){
//   return sys_exit();
// }

uint64
sys_getpid(void)
{
  return myproc()->pid;
}

uint64
sys_fork(void)
{
  return fork();
}

uint64
sys_wait(void)
{
  uint64 p;
  argaddr(0, &p);
  return wait(p);
}

uint64
sys_wait4(void)
{
  int pid;
  uint64 p;
  int options;
  argint(0,&pid);
  argaddr(1, &p);
  argint(0,&options);
  if(pid==-1)
  return wait(p);
  else
  return wait4(pid,p,options);
}

uint64
sys_sleep(void)
{
  int n;
  uint ticks0;

  argint(0, &n);
  acquire(&tickslock);
  ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(killed(myproc())){
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

uint64
sys_kill(void)
{
  int pid,sig;

  argint(0, &pid);
  argint(1, &sig);
  if(pid<=0||sig<0||sig>=SIGRTMAX){
    return -1;
  }
  return kill(pid,sig);
}

// return how many clock tick interrupts have occurred
// since start.
uint64
sys_uptime(void)
{
  uint xticks;

  acquire(&tickslock);
  xticks = ticks;
  release(&tickslock);
  return xticks;
}





//a0      a1      a2    a3    a4
//flags   stack   pid   tls   ctid
uint64 
sys_clone(void){
  uint64 stack,flags;
  uint64 ptid, tls, ctid;
  argaddr(0,&flags);
  argaddr(1,&stack);
  argaddr(2, &ptid);
  argaddr(3, &tls);
  argaddr(4, &ctid);
  if(!stack){
    return fork();
  }
  if(flags&CLONE_VM)
  return clone_thread(flags,stack,ptid,tls,ctid);
  else
  return clone(stack,flags);


}


uint64 
sys_getppid(void){
  struct proc*p=myproc();
  uint64 ppid;
  acquire(&p->lock);
  if(p->pid==1)
    ppid = 1;
  else
    ppid= p->parent->pid;
  release(&p->lock);
  //printf("ppid %d\n",ppid);
  return ppid;
}

// uint64 
// sys_spinlock_init(void){
//   uint64 addr;
//   int i;
//   argaddr(0,&addr);
//   for(i=0;i<NULOCK;i++){
//     if(ulock[i]->valid)
//       goto found;
//   }
//   return -1;
// found:
//   struct uspinlock* ulk=&ulock[i];
//   if(copyout(myproc()->pagetable,))

//   struct proc*p=myproc();
//   uint64 ppid;
//   acquire(&p->lock);
//   if(p->pid==1)
//     ppid = 1;
//   else
//     ppid= p->parent->pid;
//   release(&p->lock);
//   //printf("ppid %d\n",ppid);
//   return ppid;
// }


// uint64 
// sys_spinlock_lock(void){
//   struct proc*p=myproc();
//   uint64 ppid;
//   acquire(&p->lock);
//   if(p->pid==1)
//     ppid = 1;
//   else
//     ppid= p->parent->pid;
//   release(&p->lock);
//   //printf("ppid %d\n",ppid);
//   return ppid;
// }

// uint64 
// sys_spinlock_unlock(void){
//   struct proc*p=myproc();
//   uint64 ppid;
//   acquire(&p->lock);
//   if(p->pid==1)
//     ppid = 1;
//   else
//     ppid= p->parent->pid;
//   release(&p->lock);
//   //printf("ppid %d\n",ppid);
//   return ppid;
// }

// getuid() returns the real user ID of the calling process.
// uid_t getuid(void);
uint64 sys_getuid(void) {
    return 0;
}

uint64 sys_geteuid(void) {
    return 0;
}

uint64 sys_getgid(void) { return myproc()->pid; }

uint64 sys_getegid(void) { return 0; }

uint64 sys_set_tid_address(void){
  // uint64 address;
  // argaddr(0, &address);
  // struct proc *p = myproc();
  // p->main_thread->clear_child_tid = address;
  // int tid = p->main_thread->kt->ktid;
  // copyout(myproc()->pagetable, address, (char *)&tid, sizeof(int));

  // return tid;
  return myproc()->pid;
}

uint64 sys_gettid(void) {
  return mykthread()->ktid;
}

uint64 sys_exit_group(void) { 
  int n;
  argint(0, &n);
  exit(n);
  return 0;  // not reached
}

uint64 sys_futex(void)
{
  int futex_op, val, val3, userVal;
  uint64 uaddr, timeout, uaddr2;
  struct proc *p = myproc();
  argaddr(0, &uaddr);
  argint(1, &futex_op);
  argint(2, &val);
  argaddr(3, &timeout);
  argaddr(4, &uaddr2);
  argint(5, &val3);

  futex_op &= (FUTEX_PRIVATE_FLAG - 1);
  switch (futex_op)
  {
  case FUTEX_WAIT:
    copyin(p->pagetable, (char *)&userVal, uaddr, sizeof(int));
    // if (timeout) {
    //   if (copyin(p->pagetable, (char *)&t, timeout, sizeof(TimeSpec2)) < 0) {
    //     panic("copy time error!\n");
    //   }
    // }
    // printf("val: %d\n", userVal);
    if (userVal != val)
    {
      return -1;
    }

    futexWait(uaddr, myproc()->main_thread);
    break;
  case FUTEX_WAKE:
    futexWake(uaddr, val);
    break;
  case FUTEX_REQUEUE:
    futexRequeue(uaddr, val, uaddr2);
    break;
  default:
    panic("Futex type not support!\n");
  }
  return 0;
};
