#include "include/types.h"
#include "include/kernel/param.h"
#include "include/mm/memlayout.h"
#include "include/riscv.h"
#include "include/kernel/atomic/spinlock.h"
#include "include/defs.h"
#include "include/kernel/kthread.h"
#include "include/kernel/thread.h"
#include "include/kernel/proc.h"

struct spinlock threadlist_lock;
// struct spinlock tid_lock;
// int nexttid =1;
thread threads[THREAD_NUM];
thread *free_thread;
extern void forkret(void);

void threadinit(){
    //initlock(&tid_lock, "nexttid");
    initlock(&threadlist_lock,"threadlist");
    for(int i=0;i<THREAD_NUM;i++){
        initlock(&threads[i].lock, "thread");
        threads[i].kt=NULL;
        threads[i].p=NULL;
        threads[i].trapframe=NULL;
        threads[i].next=NULL;
        threads[i].set_child_tid=0;
        threads[i].clear_child_tid=0;
        if(i+1<THREAD_NUM){
            threads[i].next=&threads[i+1];
        }
    }
    free_thread=&threads[0];
    threads[0].prev=NULL;
}

// int alloctid()
// {
//     int tid;

//     acquire(&tid_lock);
//     tid = nexttid++;
//     release(&tid_lock);

//     return tid;
// }

thread *allocthread(struct proc *p)
{
    if (NULL == free_thread)
    {
        panic("allocthread: can not find unused kthread\n");
    }
    thread *tmp = free_thread;
    acquire(&threadlist_lock);
    free_thread = free_thread->next;
    tmp->next=NULL;
    free_thread->prev=NULL;
    
    tmp->kt = allockthread();

    acquire(&tmp->kt->lock);
    tmp->kt->p = p;
    tmp->kt->t = tmp;
    memset(&tmp->kt->context, 0, sizeof(tmp->kt->context));
    tmp->kt->context.ra = (uint64)forkret;
    tmp->kt->context.sp = tmp->kt->kstack + PGSIZE;
    release(&tmp->kt->lock);

    tmp->p = p;
    p->thread_num++;

    if ((tmp->trapframe = (struct trapframe *)kalloc()) == 0)
    {
        panic("allocthread: no mem for trapframe\n");
    }
    if (mappages(p->pagetable,TRAPFRAME(p->thread_num) , PGSIZE,(uint64)(tmp->trapframe), PTE_R | PTE_W|PTE_A|PTE_D) < 0)
        panic("thread_clone: mappages");
    tmp->vtf=TRAPFRAME(p->thread_num);
    
    #ifdef debug
    printf("alloc vtf:%p   pa:%p\n",tmp->vtf,tmp->trapframe);
    #endif

    tmp->next = p->thread_queue;
    if(p->thread_queue)
        p->thread_queue->prev=tmp;
    p->thread_queue = tmp;
    p->thread_queue->prev=NULL;
    
    release(&threadlist_lock);

    return tmp;
}

void freethread(thread*t,struct proc*p){
    // if(t->kt->state!=kt_UNUSED)
    //     freekthread(t->kt);
    // else{
    //     panic("free kthread twice\n");
    // }
    uvmunmap(t->p->pagetable,t->vtf,1,1);

    //清除线程在用户空间专属的内存区域

    if(t->kt->chan)
        panic("chan\n");

    t->killed=0;
    t->p = NULL;
    t->set_child_tid=0;
    t->clear_child_tid=0;
    t->sig_set.__val[0]=0;
    for (int i = 0; i < SIGRTMAX + 1; i++)
    {
        t->sig_pending[i] = 0;
    }

    acquire(&threadlist_lock);
    if(t==p->thread_queue){
        p->thread_queue=t->next;
        if(p->thread_queue)
            p->thread_queue->prev=NULL;
    }else{
        t->prev->next=t->next;
        if(t->next)
            t->next->prev=t->prev;
    }
    t->next=free_thread;
    free_thread->prev=t;
    t->prev=NULL;
    free_thread=t;
    release(&threadlist_lock);
}

void free_all_thread(struct proc *p)
{
    thread *tt = p->thread_queue;
    thread *ttnext = NULL;
    while (tt)
    {
        ttnext = tt->next;
        freethread(tt,p);
        tt = ttnext;
    }
    p->thread_queue=NULL;
}

void free_all_thread_except(struct proc *p,thread*t)
{
    thread *tt = p->thread_queue;
    thread *ttnext = NULL;
    while (tt)
    {
        ttnext = tt->next;
        if(tt->kt->ktid!=t->kt->ktid)
            freethread(tt,p);
        tt = ttnext;
    }
    p->thread_queue=t;t->prev=NULL; t->next=NULL;
}


/*将线程从进程的旧地址空间迁移到新地址空间*/
void transferspace(pagetable_t old,pagetable_t new,thread*t){
     uvmunmap(old,t->vtf,1,0);
    if(mappages(new,t->vtf,PGSIZE,(uint64)t->trapframe,PTE_R | PTE_W|PTE_A|PTE_D)<0)
        panic("transferthread: mappages");
    //修改线程在用户空间专属的内存区域
}


/*将某个线程从一个进程迁移到新的进程*/
void transferproc(struct proc*old,struct proc*new,thread*t){
    if(t->p!=old){
        panic("transferthread:thread is not in old proc\n");
    }
    acquire(&threadlist_lock);
    thread*tt=old->thread_queue;   //得修改
    thread* prev=old->thread_queue;
    while(tt){
        if(tt==t){
            if(tt==old->thread_queue){
                old->thread_queue=tt->next;
                break;
            }
            prev->next=tt->next;
            break;
        }
        prev=tt;
        tt=tt->next;
    }
    t->p=new;
    t->next=new->thread_queue;
    new->thread_queue=t;
    release(&threadlist_lock);
    uvmunmap(old->pagetable,t->vtf,1,0);
    if(mappages(new->pagetable,t->vtf,PGSIZE,(uint64)t->trapframe,PTE_R | PTE_W|PTE_A|PTE_D)<0)
        panic("transferthread: mappages");
    //修改线程在用户空间专属的内存区域
}

