#!/bin/bash
#因为目录过于复杂，此脚本用于检查includepath是否正确，并自动改正
target_dir="../"
include_dir="../include/"

# 查找并处理每个C文件
find "$target_dir" -name '*.c' -or -name '*.h' | while read cfile; do
    echo "Processing $cfile"
    # 读取文件中所有#include "..."行
    grep '#include "' "$cfile" | while IFS='"' read -r _ include_path _; do
        # 完全路径
        full_include_path=$(realpath --relative-to="$target_dir" "$include_path" 2>/dev/null || echo "$include_path")
        
        # 检查文件是否存在
        if [[ ! -f "$target_dir/$full_include_path" && ! -f "$full_include_path" ]]; then
            echo "Invalid path: $include_path in file $cfile"
            # 尝试在include_dir中找到正确的文件
            correct_path=$(find "$include_dir" -type f -name "$(basename "$include_path")" -print -quit)
            if [[ -n $correct_path ]]; then
                # 计算相对路径
                correct_relative_path=$(realpath --relative-to="$target_dir" "$correct_path")
                # 替换文件中的错误路径
                sed -i "s~#include \"$include_path\"~#include \"$correct_relative_path\"~" "$cfile"
                echo "Corrected to: #include \"$correct_relative_path\" in file $cfile"
            else
                echo "No valid path found for $include_path"
            fi
        fi
    done
done
